@@ -30,7 +30,7 @@ class Globals
 {
 	// These config options are modified by ChangeAppsettings.sh script - see the detailed descriptions there
 	public static String ApplicationName = "Tucnak";
-	public static String AppLibraries[] = { "sdl-1.2", "glib-2.0", "gthread-2.0", "pngfkk", "usb", "ftdi", "zia", "iconv", "hamlib", "sndfile"};
+	public static String AppLibraries[] = { "sdl-1.2", "glib-2.0", "gthread-2.0", "pngfkk", /*"usb", "ftdi",*/ "iconv", "zia", "hamlib", "sndfile"};
 	public static final boolean Using_SDL_1_3 = false;
 	public static String DataDownloadUrl = "Data files are 2 Mb|https://sourceforge.net/projects/libsdl-android/files/CommanderGenius/commandergenius-data.zip/download^High-quality GFX and music - 40 Mb|https://sourceforge.net/projects/libsdl-android/files/CommanderGenius/commandergenius-hqp.zip/download";
 	public static int VideoDepthBpp = 16;
