@@ -1,6 +1,6 @@
 /*
     ac.c - aircraft trace
-    Copyright (C) 2013-2016 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2013-2021 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
 #include "qrvdb.h"
 #include "qsodb.h"
 #include "rc.h"
+#include "rotar.h"
 #include "session.h"
 #include "subwin.h"
 #include "tsdl.h"
@@ -142,6 +143,9 @@ void acs_load(struct acs *acs, const char *data){
 
         c = strtok_r(NULL, AC_SEP, &token_ptr); // "C87EE8"
         if (!c) goto x;
+        if (!*c) goto x;
+		g_strlcpy(ac->id, c+1, AC_ID_MAXLEN);
+
         c = strtok_r(NULL, AC_SEP, &token_ptr); // latitude
         if (!c) goto x;
         ac->w = atof(c) * M_PI / 180.0;
@@ -152,7 +156,7 @@ void acs_load(struct acs *acs, const char *data){
 
         c = strtok_r(NULL, AC_SEP, &token_ptr); // azimuth in degrees
         if (!c) goto x;
-        ac->qtf = (atoi(c) - 90) * M_PI / 180.0;
+        ac->qtfdir = (atoi(c) - 90) * M_PI / 180.0;
 
         c = strtok_r(NULL, AC_SEP, &token_ptr);  // asl in feets
         if (!c) goto x;
@@ -221,7 +225,7 @@ void acs_load2(struct acs *acs, const char *data){
         ac->h = atof(c) * M_PI / 180.0;
         c = strtok_r(NULL, AC_SEP, &token_ptr);
         if (!c) goto x;
-        ac->qtf = (atoi(c) - 90) * M_PI / 180.0;
+        ac->qtfdir = (atoi(c) - 90) * M_PI / 180.0;
         c = strtok_r(NULL, AC_SEP, &token_ptr);
         if (!c) goto x;
         ac->asl = atoi(c) * 0.3048;
@@ -253,7 +257,7 @@ x:;
 
 // thread
 void ac_compute(struct acs *acs, struct ac *ac, struct ac_counterpart *ctp, struct ac_result *res){
-    double h, w, qrb, qtfrad;
+    double h, w, qrb, qtf;
     int i, ret, retin, ii, ari;
     time_t now, dt;
     double h2, w2, ikx, iky, iqrb, iqtf, minasl;
@@ -271,17 +275,17 @@ void ac_compute(struct acs *acs, struct ac *ac, struct ac_counterpart *ctp, stru
         res->start = (time_t)0;
         return;
     }
-    qrbqtf2hw(ac->h, ac->w, ac->speed * dt / 3600.0, ac->qtf + (90 * M_PI / 180.0), &h, &w);
-    hw2qrbqtf(acs->th.myh, acs->th.myw, h, w, &qrb, &qtfrad); 
-    ac->kx = (int)(  qrb * sin(qtfrad));
-    ac->ky = (int)(- qrb * cos(qtfrad));
+    qrbqtf2hw(ac->h, ac->w, ac->speed * dt / 3600.0, ac->qtfdir + (90 * M_PI / 180.0), &h, &w);
+    hw2qrbqtf(acs->th.myh, acs->th.myw, h, w, &qrb, &ac->qtf); 
+	ac->kx = (int)round(qrb * sin(ac->qtf));
+	ac->ky = (int)round(-qrb * cos(ac->qtf));
    // dbg("%5.3f %5.3f -> %5.3f %5.3f = %f %f %f | %d %d\n", mh, mw, ac->h, ac->w, qrb, qtfrad, (qtfrad * 180/M_PI + 90), ac->kx, ac->ky);
     //dbg("qtf=%f %f speed=%d\n", ac->qtf, ac->qtf * 180.0 / M_PI, ac->speed);
     
-    qrbqtf2hw(h, w, ac->speed * 10.0 / 60.0 , ac->qtf + (90 * M_PI / 180.0), &h2, &w2);
-    hw2qrbqtf(acs->th.myh, acs->th.myw, h2, w2, &qrb, &qtfrad);
-    ac->pakx = (int)(  qrb * sin(qtfrad));
-    ac->paky = (int)(- qrb * cos(qtfrad));
+	qrbqtf2hw(h, w, ac->speed * 10.0 / 60.0, ac->qtfdir + (90 * M_PI / 180.0), &h2, &w2);
+	hw2qrbqtf(acs->th.myh, acs->th.myw, h2, w2, &qrb, &qtf);
+	ac->pakx = (int)round(qrb * sin(qtf));
+	ac->paky = (int)round(-qrb * cos(qtf));
     //dbg("h=%f  w=%f | kx=%d  ky=%d \n", h * 180.0/M_PI, w * 180.0/M_PI, ac->kx[i], ac->ky[i]);
 
 #ifdef Z_HAVE_SDL
@@ -346,7 +350,7 @@ void ac_compute(struct acs *acs, struct ac *ac, struct ac_counterpart *ctp, stru
     res->stop += now;
 
     res->now = now;
-    res->angle = ctp->qtfrad - (ac->qtf + M_PI/2);
+	res->angle = ctp->qtfrad - (ac->qtfdir + M_PI / 2);
     while (res->angle < 0.0) res->angle += 2 * M_PI;
     while (res->angle > 2 * M_PI) res->angle -= 2 * M_PI;
     if (res->angle > M_PI) res->angle = res->angle - M_PI;
@@ -446,12 +450,12 @@ void ac_zorder(struct subwin *sw, struct ac *ac){
     r = 16;
     rad = atan2((double)(ac->paky - ac->ky), (double)(ac->pakx - ac->kx));
 
-    vx  = (int)(px + 0.6*r * cos(rad) + 0.5);
-    vy  = (int)(py + 0.6*r * sin(rad) + 0.5);
-    vx1 = (int)(px - 0.4*r * cos(rad + 0.6) + 0.5);
-    vy1 = (int)(py - 0.4*r * sin(rad + 0.6) + 0.5);
-    vx2 = (int)(px - 0.4*r * cos(rad - 0.6) + 0.5);
-    vy2 = (int)(py - 0.4*r * sin(rad - 0.6) + 0.5);
+    vx  = (int)round(px + 0.6*r * cos(rad) + 0.5);
+    vy  = (int)round(py + 0.6*r * sin(rad) + 0.5);
+    vx1 = (int)round(px - 0.4*r * cos(rad + 0.6) + 0.5);
+    vy1 = (int)round(py - 0.4*r * sin(rad + 0.6) + 0.5);
+    vx2 = (int)round(px - 0.4*r * cos(rad - 0.6) + 0.5);
+    vy2 = (int)round(py - 0.4*r * sin(rad - 0.6) + 0.5);
 
     undercursor = 0;
     if (sw->mx >= z_min3_d(vx, vx1, vx2) && 
@@ -489,12 +493,12 @@ void plot_ac(struct subwin *sw, struct ac *ac){
 
     rad = atan2((double)(ac->paky - ac->ky), (double)(ac->pakx - ac->kx));
     
-    vx  = (int)(px + 0.6*r * cos(rad) + 0.5);
-    vy  = (int)(py + 0.6*r * sin(rad) + 0.5);
-    vx1 = (int)(px - 0.4*r * cos(rad + 0.6) + 0.5);
-    vy1 = (int)(py - 0.4*r * sin(rad + 0.6) + 0.5);
-    vx2 = (int)(px - 0.4*r * cos(rad - 0.6) + 0.5);
-    vy2 = (int)(py - 0.4*r * sin(rad - 0.6) + 0.5);
+    vx  = (int)round(px + 0.6*r * cos(rad) + 0.5);
+    vy  = (int)round(py + 0.6*r * sin(rad) + 0.5);
+    vx1 = (int)round(px - 0.4*r * cos(rad + 0.6) + 0.5);
+    vy1 = (int)round(py - 0.4*r * sin(rad + 0.6) + 0.5);
+    vx2 = (int)round(px - 0.4*r * cos(rad - 0.6) + 0.5);
+    vy2 = (int)round(py - 0.4*r * sin(rad - 0.6) + 0.5);
 
 
     undercursor = 0;
@@ -502,11 +506,13 @@ void plot_ac(struct subwin *sw, struct ac *ac){
         sw->my >= z_min3_d(vy, vy1, vy2) &&
         sw->mx <= z_max3_d(vx, vx1, vx2) &&
         sw->my <= z_max3_d(vy, vy1, vy2)) undercursor = 1;
-
     if (undercursor){
         fcolor = z_makecol(21, 193, 106);
         bcolor = z_makecol(15, 150, 70);
-    }else if (ctp->path_valid == 1 && res->under){
+	}else if (strcmp(ac->id, gacs->sh.tracked) == 0){
+		fcolor = z_makecol(255, 0, 0);
+		bcolor = z_makecol(192, 0, 0);
+	}else if (ctp->path_valid == 1 && res->under){
         fcolor = z_makecol(128, 128, 128);
         bcolor = z_makecol(64, 64, 64);
     }else if (ctp->path_valid == 1 && res->crossing){
@@ -523,12 +529,12 @@ void plot_ac(struct subwin *sw, struct ac *ac){
     z_line(sw->screen, vx1, vy1, vx2, vy2, fcolor);
     z_putpixel(sw->screen, px, py, fcolor);
 
-    vx  = (int)(px + 0.6*r * cos(rad));
-    vy  = (int)(py + 0.6*r * sin(rad));
-    vx1 = (int)(px - 0.4*r * cos(rad + 0.6));
-    vy1 = (int)(py - 0.4*r * sin(rad + 0.6));
-    vx2 = (int)(px - 0.4*r * cos(rad - 0.6));
-    vy2 = (int)(py - 0.4*r * sin(rad - 0.6));
+    vx  = (int)round(px + 0.6*r * cos(rad));
+    vy  = (int)round(py + 0.6*r * sin(rad));
+    vx1 = (int)round(px - 0.4*r * cos(rad + 0.6));
+    vy1 = (int)round(py - 0.4*r * sin(rad + 0.6));
+    vx2 = (int)round(px - 0.4*r * cos(rad - 0.6));
+    vy2 = (int)round(py - 0.4*r * sin(rad - 0.6));
 
     if (undercursor)    
     {
@@ -773,6 +779,9 @@ gpointer acs_thread(gpointer arg){
         char *data;
         struct ac *ac;
         int i, j;
+		char tracked[AC_ID_MAXLEN];
+		int rotnr;
+
 
         for (i = 0; i < 10; i++) {
             if (acs->sh.new_tmpctp) break;
@@ -815,6 +824,8 @@ gpointer acs_thread(gpointer arg){
             acs->sh.new_infoctp = 0;
             new_infoctp = 1;
         }
+		g_strlcpy(tracked, acs->sh.tracked, AC_ID_MAXLEN);
+		rotnr = acs->sh.rotnr;
         MUTEX_UNLOCK(acs->sh);
 
         if (acs->sh.acs){
@@ -827,9 +838,15 @@ gpointer acs_thread(gpointer arg){
                 new_infoctp = 0;
             }
             //ST_START();
+			int qtf = -360;
             for (i = 0; i < acs->sh.acs->len; i++){
                 ac = (struct ac *)g_ptr_array_index(acs->sh.acs, i);
                 ac_compute(acs, ac, &acs->tmpqso, &ac->tmpres);
+				if (strlen(ac->id) > 0 && strcmp(ac->id, tracked) == 0){
+					qtf = (int)round(ac->qtf * 180.0 / M_PI);
+					dbg("qtf=%d\n", qtf);
+					//if (qtf >= 360) qtf -= 360;
+				}
                 if (acs->thread_break) break;
             }
             for (i = 0; i < acs->sh.acs->len; i++){
@@ -899,7 +916,23 @@ next:;
                 MUTEX_UNLOCK(qrv->qrvs);
                 if (acs->thread_break) break;
             }
-            //ST_STOP("ac_recompute");
+
+			if (qtf > -360){
+				MUTEX_LOCK(acs->sh);
+#define HYSTERESIS 1
+				if (strlen(acs->sh.tracked) > 0){ // prevent race condition when ac_track(NULL) was called during for { ac_compute }
+					struct rotar *rot = get_rotar(rotnr);
+					if (rot->type != ROT_REMOTE /* not sure if thread-safe */) {
+						//dbg("rot_seek(%d, %d)\n", rotnr, qtf);
+						if (abs(rot->qtf - qtf) > HYSTERESIS){
+							rot_seek(rot, qtf, -90);
+						}
+					}
+				}
+				MUTEX_UNLOCK(acs->sh);
+
+			}
+			//ST_STOP("ac_recompute");
             zselect_msg_send(zsel, "AC;r"); 
         }
 
@@ -907,11 +940,12 @@ next:;
     return NULL;
 }
 
-void acs_update_qth(struct acs *acs, struct subwin *sw){
+void acs_update_qth(struct acs *acs){
     if (!acs) return;
+	if (!gses) return;
 
     MUTEX_LOCK(acs->sh);
-    if (acs->sh.myh != sw->myh || acs->sh.myw != sw->myw){
+    if (acs->sh.myh != gses->myh || acs->sh.myw != gses->myw){
         MUTEX_UNLOCK(acs->sh);
         free_acs(acs);
         gacs = init_acs();
@@ -1061,5 +1095,41 @@ int acs_get_wingspan(struct acs *acs, char *icao){
     return 0;
 }
 
+struct ac *ac_under(struct acs *acs, struct subwin *sw){
+	struct ac *ac, *ret = NULL;
+	int i;
+
+	if (!gacs) return NULL;
+
+	MUTEX_LOCK(acs->sh);
+	if (acs->sh.acs){
+		for (i = 0; i < acs->sh.acs->len; i++){
+			ac = (struct ac *)g_ptr_array_index(acs->sh.acs, i);
+			ac_zorder(sw, ac);
+			if (ac->zorder == ACZO_CURSOR){
+				ret = ac;
+				break;
+			}
+		}
+	}
+	MUTEX_UNLOCK(acs->sh);
+	return ret;
+}
+
+void ac_track(struct acs *acs, char *id, int rotnr){
+	if (!acs) return;
+
+	MUTEX_LOCK(acs->sh);
+
+	if (id == NULL){
+		g_strlcpy(acs->sh.tracked, "", AC_ID_MAXLEN);
+		acs->sh.rotnr = -1;
+	}else{
+		g_strlcpy(acs->sh.tracked, id, AC_ID_MAXLEN);
+		acs->sh.rotnr = rotnr;
+	}
+
+	MUTEX_UNLOCK(acs->sh);
+}
 
 
