@@ -1,6 +1,5 @@
-/*
-    Tucnak - VHF contest log
-    Copyright (C) 2002-2016  Ladislav Vaiz <ok1zia@nagano.cz>
+/*  Tucnak - VHF contest log
+    Copyright (C) 2002-2018  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -18,6 +17,7 @@
 #include "language2.h"
 #include "oss.h"
 #include "pa.h"
+#include "rtlsdr.h"
 #include "rc.h"
 #include "sndf.h"
 #include "sndpipe.h"
@@ -32,7 +32,7 @@ struct dsp *init_dsp(enum dsp_type type, enum dsp_cfg_group dcg){
 
     dsp = g_new0(struct dsp, 1);   
     dsp->type = type;
-#ifdef HAVE_OSS
+#if defined(HAVE_OSS) || defined(HAVE_SWRADIO)
     dsp->fd = -1;
 #endif
 #if (!defined(HAVE_OSS)) && (!defined(HAVE_ALSA) && !defined(HAVE_SNDFILE))
@@ -213,6 +213,26 @@ struct dsp *init_dsp(enum dsp_type type, enum dsp_cfg_group dcg){
 #endif
 			dsp->set_sdr_format = sdrc_set_sdr_format;
 			break;
+        case DSPT_SWRADIO:
+#ifdef HAVE_SWRADIO
+            dsp->open   = swradio_open;
+            dsp->close2 = swradio_close;
+            dsp->read   = swradio_read;
+#else
+            log_addf("No support for /dev/swradio");
+#endif
+            break;
+        case DSPT_RTLSDR:
+#ifdef HAVE_LIBRTLSDR
+            dsp->open   = rtlsdr_open2;
+            dsp->close2 = rtlsdr_close2;
+            dsp->read   = rtlsdr_read;
+			dsp->set_sdr_format = rtlsdr_set_sdr_format;
+			dsp->rtlsdr_device = g_strdup("0");
+#else
+            log_addf("No support for librtlsdr");
+#endif
+            break;
         default:
             zinternal("bad dsp->type");
             break;
@@ -260,6 +280,10 @@ void free_dsp(struct dsp *dsp){
 	zg_free0(dsp->sfinfo);
 	zg_free0(dsp->play_filename);
 	zg_free0(dsp->rec_filename);
+#endif
+#ifdef HAVE_LIBRTLSDR
+	zg_free0(dsp->rtlsdr_device);
+	zg_free0(dsp->rtlsdr_buf);
 #endif
     zg_free0(dsp->name);
     zg_free0(dsp->source);
