@@ -587,7 +587,9 @@ int sw_dxc_mouse_func(struct subwin *sw, struct event *ev, int fw){
                 return 1;
             }
 			g_free(sw->callunder);
-			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y, &sw->qrg);
+			g_free(sw->refunder);
+			sw->refunder = NULL;
+			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y, &sw->qrg, &(sw->refunder));
 			if (!sw->callunder) break;
 			sw_dxc_toggle_highlight(sw, sw->callunder, 1);
             return 1;
@@ -600,7 +602,9 @@ int sw_dxc_mouse_func(struct subwin *sw, struct event *ev, int fw){
                 return 1;
             }
 			g_free(sw->callunder);
-			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y, &sw->qrg);
+			g_free(sw->refunder);
+			sw->refunder = NULL;
+			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y, &sw->qrg, &(sw->refunder));
 			if (!sw->callunder) break;
 			mi = new_menu(1);
 			mi->rtext = sw->callunder;
@@ -977,13 +981,29 @@ static void dxc_parse_qrg(char *s, double *out_qrg){
 	}
 }
 
-
 #define FREE_SW_CX if (c1) {g_free(c1); c1=NULL;} if (c2) {g_free(c2); c2=NULL;}
+#define REF_MAX_LEN 12
 
-char *sw_dxc_call_under(struct subwin *sw, int x, int y, double *out_qrg){
+static int dxc_parse_ref(char *s, char *out_ref){
+	int match, ret = 0;
+	char *c1 = NULL, *c2 = NULL;
+	if (out_ref != NULL){
+		c1 = NULL;
+		c2 = NULL;
+		match = regmatch(s, "[A-Z0-9]{1,4}/[A-Z]{2}-[0-9]{3}", &c1, &c2, NULL);
+		if (match == 0 && c1){
+			safe_strncpy0(out_ref, c1, REF_MAX_LEN);
+			ret = 1;
+		}
+		FREE_SW_CX;
+	}
+	return ret;
+}
+
+char *sw_dxc_call_under(struct subwin *sw, int x, int y, double *out_qrg, char** out_ref){
 	int index;
     int ret, len, i1, i2;
-    char *c1 = NULL, *c2 = NULL, *s, call[25];
+    char *c1 = NULL, *c2 = NULL, *s, call[25], ref[REF_MAX_LEN] = {0};
 
 	//dbg("sw_dxc_call_under(sw, %d, %d)\n", x, y);
 	if (out_qrg != NULL) *out_qrg = 0.0;
@@ -1061,6 +1081,10 @@ char *sw_dxc_call_under(struct subwin *sw, int x, int y, double *out_qrg){
 		break;
 	}
 	if (i2 > i1 && i2 - i1 < 20){
+		if (out_ref && dxc_parse_ref(s, ref))
+		{
+			*out_ref = g_strdup(ref);
+		}			
 		g_strlcpy(call, s + i1, Z_MIN(i2 - i1 + 1, 20));
 		z_strip_from(call, '-'); //rbn
 		if (z_can_be_call(call, ZCBC_SLASH | ZCBC_MINUS)){
@@ -1090,6 +1114,10 @@ doit:;
    /* dbg("ret=%d  c1='%s'  c2='%s'\n", ret, c1, c2);*/
     FREE_SW_CX;
 	dxc_parse_qrg(s, out_qrg);
+	if (out_ref && dxc_parse_ref(s, ref))
+	{
+		*out_ref = g_strdup(ref);
+	}
     g_free(s);
     if (ret) return NULL;
 
@@ -1165,6 +1193,7 @@ void sw_dxc_use(void *itdata, void *menudata){
 	if (!aband) return;
 	if (aband->readonly) return;
 	process_input(aband, sw->callunder, 0);
+	if (sw->refunder) process_input(aband, sw->refunder, 0);
 	il_add_to_history(INPUTLN(aband), sw->callunder);
 	sw_unset_focus();
     il_set_focus(INPUTLN(aband));
