@@ -13,7 +13,7 @@
 
 #include "header.h"
 
-#if defined(HAVE_LIBFFTW3) && defined(Z_HAVE_SDL)
+#ifdef USE_SDR
 
 
 #include "ssbd.h"
@@ -42,9 +42,11 @@ struct fft{
     int n;
     int samplerate;
     int pal[FFT_COLORS];
+#ifdef Z_HAVE_SDL    
     SDL_Surface *screen;
     MUTEX_DEFINE(screen);
-    int screeny;    
+    int screeny; 
+#endif   
     double *amp;
 	int amplen;
 #ifdef USE_NEA
