@@ -23,11 +23,14 @@
 #include "zpng.h"
 
 #define HTML_QSO_STR(item) zg_string_eprintfa("h", gs, "<td>%s </td> ", item);
+#define HTML_QSO_STR_R(item) zg_string_eprintfa("h", gs, "<td align=\"right\">%s </td> ", item);
 #define HTML_QSO_INT(item) g_string_append_printf(gs, "<td align=\"right\">%d </td> ", item);
 
 #define HTML_HDR(item) g_string_append_printf(gs, "<th bgcolor=\"#e4b97c\"><font color=\"#800000\">%s</font></th>\n", item);
 
 static int bandrefi = 0;
+static int total_qsos = 0;
+static int total_points = 0;
 gchar *qh(GString *gs, gchar *s){
 	char *c;
 
@@ -159,13 +162,24 @@ void html_band_ref(GString *gs, struct band *b, struct config_band *confb){
         g_string_append_printf(gs, "<tr bgcolor=\"#606060\">\n");
     zg_string_eprintfa("h", gs, "<td><a href=\"#%c\">%s</a></td>\n", b->bandchar, b->bandname);
     g_string_append_printf(gs, "<td align=\"right\">%d</td>\n", b->stats->nqsos);
+    total_qsos += b->stats->nqsos;
     g_string_append_printf(gs, "<td align=\"right\">%d</td>\n", b->stats->ntotal);
+    total_points += b->stats->ntotal;
     if (b->stats->nqsos){ // happened always
         g_string_append_printf(gs, "<td align=\"right\">%5.1f</td>\n", (double)b->stats->ntotal/(double)b->stats->nqsos);
     }
     g_string_append_printf(gs, "</tr>\n");
 }
     
+void html_total_sum(GString *gs, struct band *b, struct config_band *confb){
+    if (bandrefi++%2)
+        g_string_append_printf(gs, "<tr bgcolor=\"#505050\">\n");
+    else
+        g_string_append_printf(gs, "<tr bgcolor=\"#606060\">\n");
+    g_string_append_printf(gs, "<td></td><td align=\"right\"><b>%d</b></td><td align=\"right\"><b>%d</b></td><td></td></tr>\n", total_qsos, total_points);
+    g_string_append_printf(gs, "</tr>\n");
+}
+
 void html_ref_footer(GString *gs, struct band *b, struct config_band *confb){
     g_string_append_printf(gs, "</table>\n");
 }
@@ -219,8 +233,10 @@ void html_band_header(GString *gs, struct band *b, struct config_band *confb, in
 	HTML_HDR("MODE");
 	HTML_HDR("TIME");
 	HTML_HDR("CALLSIGN");
-	HTML_HDR("TX");
-	HTML_HDR("RX");
+	HTML_HDR("RSTS");
+	HTML_HDR("STX");
+	HTML_HDR("RSTR");
+	HTML_HDR("SRX");
 	HTML_HDR("LOCATOR");
 	HTML_HDR("QRB");
 	HTML_HDR("OPERATOR");
@@ -275,7 +291,7 @@ void html_qsos(GString *gs, struct band *b, struct config_band *confb){
     j=0;
     for (i=0; i<b->qsos->len; i++){
         char time[10];
-        gchar *c;
+        //gchar *c;
         
         q = get_qso(b, i); 
         if (q->error) continue;
@@ -289,18 +305,16 @@ void html_qsos(GString *gs, struct band *b, struct config_band *confb){
         time[2]='.'; time[3]=q->time_str[2]; time[4]=q->time_str[3]; time[5]='\0';
         
 		HTML_QSO_STR(q->date_str);
-     	HTML_QSO_STR(html_modes[abs(q->mode)%10]);
+		HTML_QSO_STR(html_modes[abs(q->mode)%10]);
 		HTML_QSO_STR(time);
-        HTML_QSO_STR(q->callsign);
+		HTML_QSO_STR(q->callsign);
 
-        c = g_strdup_printf("%s %s", q->rsts, q->qsonrs);
-		HTML_QSO_STR(c);
-        g_free(c);
+		HTML_QSO_STR_R(q->rsts);
+		HTML_QSO_STR_R(q->qsonrs);
+        
+		HTML_QSO_STR_R(q->rstr);
+		HTML_QSO_STR_R(q->qsonrr);
         
-        c = g_strdup_printf("%s %s", q->rstr, q->qsonrr);
-		HTML_QSO_STR(c);
-        g_free(c);
-
 		HTML_QSO_STR(q->locator);
 		HTML_QSO_INT((int)q->qrb);
 		HTML_QSO_STR(q->operator_);
@@ -313,8 +327,8 @@ void html_qsos(GString *gs, struct band *b, struct config_band *confb){
 }
 
 void html_complete(GString *gs, int flags){
-    struct band *band;
-    struct config_band *confb;
+    struct band *band = NULL;
+    struct config_band *confb = NULL;
     int i;
     GString *gs2, *title;
     int header_saved=0;
@@ -334,6 +348,7 @@ void html_complete(GString *gs, int flags){
 	chart = sw_raise_or_new(SWT_CHART);
 
 	bandrefi = 0;
+	total_qsos = total_points = 0;
     for (i=0; i<ctest->bands->len; i++){
         band = (struct band*)g_ptr_array_index(ctest->bands, i);
         confb = (struct config_band*)get_config_band_by_bandchar(band->bandchar);
@@ -383,7 +398,11 @@ void html_complete(GString *gs, int flags){
 
         html_band_ref(gs, band, confb);
     }
-    html_ref_footer(gs, band, confb);
+
+	if (band != NULL && confb != NULL){
+		html_total_sum(gs, band, confb);
+		html_ref_footer(gs, band, confb);
+	}
 
     for (i=0; i<ctest->bands->len; i++){
         band = (struct band*)g_ptr_array_index(ctest->bands, i);
