@@ -12,6 +12,7 @@
 #include "header.h"
 
 #include "cwdaemon.h"
+#include "fifo.h"
 #include "inputln.h"
 #include "kbdbind.h"
 #include "kbd.h"
@@ -359,6 +360,29 @@ show_history:
                     g_free(clipboard);*/
                     goto dsp_f;
                 }
+				case ACT_AC_INFO:
+					if (il->readonly) break;
+					il->cdata[il->cpos] = 0;
+					if (il->band) break;
+					int i;
+					for (i = il->history->len - 1; i >= 0; i--){
+						char *c, *c1 = NULL, *c2 = NULL, *c3 = NULL;
+						c = (char *)g_ptr_array_index(il->history, i);
+						if (!c) continue; /* probably unreached */
+						if (regmatch(c, "^/[cC][qQ] ([a-zA-Z0-9/-]{3,14})", &c1, &c2, &c3, NULL) == 0){
+							dbg("c1=%s c2=%s c3=%s\n", c1, c2, c3);
+							g_free(gses->callunder);
+							gses->callunder = g_strdup(c2);
+							FREE_Cx;
+							if (!gses->callunder || !*gses->callunder) break;
+							z_str_uc(gses->callunder);
+							fifo_kst_ac_info(NULL, NULL);
+							break;
+						}
+						FREE_Cx;
+					}
+					goto dsp_f;
+					break;
                 default:
                     if (cfg->slashkey && *cfg->slashkey){
                         if (ev->x == '/') ev->x = *cfg->slashkey;
