@@ -14,7 +14,7 @@
 #include "header.h"
 #include "bfu.h"
 
-#define VALID_CHARS " ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/#',.-_:?!;"
+#define VALID_CHARS " ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/#',.+-_:?!;*"
 
 struct inputln {
     int (*fn)(struct dialog_data *, struct dialog_item_data *);
