@@ -1187,17 +1187,29 @@ char *sw_kst_counterp(struct subwin *sw, char *line, int *color){ // thread unsa
     *color = 0;
     if (strlen(line) < 20) return 0;
     c = z_strcasestr(line, "> (");
-    if (!c) return NULL;
-    if (strncmp(c, "> (0) (", 7) == 0){ /* "> (0) (CALL) ..." */
-        c += 4;  /* "CALL) ..." */
-    }
-    g_strlcpy(call, c + 3, sizeof(call));
-    z_strip_from(call, ')');
-    z_get_raw_call(raw, call);
-    
-    z_str_uc(raw);
-    *color = sw_kst_color(raw);
-    return call;
+	if (c) {
+		if (strncmp(c, "> (0) (", 7) == 0){ /* "> (0) (CALL) ..." */
+			c += 4;  /* "CALL) ..." */
+		}
+		g_strlcpy(call, c + 3, sizeof(call));
+		z_strip_from(call, ')');
+		z_get_raw_call(raw, call);
+
+		z_str_uc(raw);
+		*color = sw_kst_color(raw);
+		return call;
+	}
+	c = z_strcasestr(line, "> ");
+	if (c) {
+		g_strlcpy(call, c + 2, sizeof(call));
+		z_strip_from(call, ' ');
+		z_get_raw_call(raw, call);
+
+		z_str_uc(raw);
+		*color = sw_kst_color(raw);
+		return call;
+	}
+	return NULL;
 }
 
 int sw_kst_color(char *call){
