@@ -563,7 +563,7 @@ void settings(GString *gs){
 
 static void init(void)
 {
-	char *c;
+	char *c = NULL;
 	char userbuf[64];
 
     parse_options(ac, av);
@@ -701,6 +701,7 @@ static void init(void)
     ntpq = init_ntpq();        
 #endif
 #ifdef HAVE_HAMLIB
+
     init_trigs();        
 #endif
 #ifdef USE_FFT
@@ -712,7 +713,7 @@ static void init(void)
 #endif
 #ifdef Z_HAVE_SDL
 	gbat = zbat_init();
-	grain = init_rain();
+	//grain = init_rain();
 #endif
 
     
@@ -873,6 +874,7 @@ void redraw(void){
 
 int main(int argc, char *argv[])
 {
+
 	//zavgfilter_test();
 #if 0
     GString *gs = g_string_sized_new(8000);
@@ -1164,11 +1166,12 @@ int main(int argc, char *argv[])
 #ifdef __MINGW32__
     init_mingw();
 #endif
+
     zsel = zselect_init(redraw);
 	zselect_msg_set(zsel, main_msg_handler);
 	//zselect_start_profile(zsel, 0.099);
-    init();
-    zselect_loop(zsel);
+	init();
+	zselect_loop(zsel);
     terminate_all_subsystems();
 
     sound(0);
