@@ -138,19 +138,26 @@ void zoom(struct subwin *sw, double factor, int centerx, int centery){
 	}
 	sw->gdirty=1;
     redraw_later();
-    dbg("zoom=%d\n", sw->zoom);
-
-	int size = 512;
-	double qrb, qtf, dh, dw;
-	px2km(sw, size, size, &kx, &kx);
-	km2qrbqtf(kx, ky, &qrb, &qtf);
-	qrbqtf2hw(sw->myh, sw->myw, qrb, qtf, &dh, &dw);
-
-	/*int gz =*/ tile_googlezoom(size, dh, dw, size, size);
+    //dbg("zoom=%d\n", sw->zoom);
 }
 
+void map_turn_rot(void *itdata, void *menudata){
+	struct subwin *sw = (struct subwin *)menudata;
+	int nr = GPOINTER_TO_INT(itdata);
+	struct rotar *rot = get_rotar(nr);
+	ac_track(gacs, NULL, -10);
+	rot_seek(rot, sw->uhel, -90);
+}
+
+void map_track(void *itdata, void *menudata){
+	struct subwin *sw = (struct subwin *)menudata;
+	int nr = GPOINTER_TO_INT(itdata);
+	ac_track(gacs, sw->acunder, nr);
+}
+
+
 int sw_map_mouse_func(struct subwin *sw, struct event *ev, int fw){
-    int dx,dy, lmx, lmy;
+    int dx,dy, lmx = -1, lmy = -1;
     double stepd;
     
     if (!sdl) return 0;
@@ -223,6 +230,8 @@ int sw_map_mouse_func(struct subwin *sw, struct event *ev, int fw){
     }
     
     switch (ev->b & BM_EBUTT){
+		struct menu_item *mi;
+
         case B_LEFT:
     /*        dbg("left %d %d\n", ev->mx, ev->my);*/
             sw->olddragx=ev->mx;
@@ -232,7 +241,39 @@ int sw_map_mouse_func(struct subwin *sw, struct event *ev, int fw){
         /*    dbg("middle\n");*/
             break;
         case B_RIGHT:
-            /*dbg("right\n");*/
+			/*dbg("right\n");*/
+			if (lmx < 0) break;
+			mi = new_menu(1);
+			mi->rtext = TRANSLATE("Turn antenna");
+			MUTEX_LOCK(rotars);
+			int i = 0;
+			for (i = 0; i<rotars->len; i++){
+				struct rotar *rot = (struct rotar *) g_ptr_array_index(rotars, i);
+				add_to_menu(&mi, rot->desc, "", "", MENU_FUNC map_turn_rot, GINT_TO_POINTER(i), 0);
+			}
+			struct ac *ac = ac_under(gacs, sw);
+			g_free(sw->acunder);
+			sw->acunder = NULL;
+			if (ac){
+				sw->acunder = g_strdup(ac->id);
+				for (i = 0; i<rotars->len; i++){
+					struct rotar *rot = (struct rotar *) g_ptr_array_index(rotars, i);
+					add_to_menu(&mi, "Track airplane", rot->rotstr, rot->rotstr, MENU_FUNC map_track, GINT_TO_POINTER(i), 0);
+				}
+			}
+
+			MUTEX_UNLOCK(rotars);
+			if (i == 0) break;
+			set_window_ptr(gses->win, ev->x - 3, ev->y + 1);
+
+			int kx, ky;
+			double qrb, qtf;
+			px2km(sw, lmx, lmy, &kx, &ky);
+			km2qrbqtf(kx, ky, &qrb, &qtf);
+			int uhel = (int)round(qtf * 180 / M_PI);
+			sw->uhel = uhel;
+			do_menu(mi, sw);
+
             break;
         case B_WHUP:
             /*dbg("wheel up\n");*/
@@ -323,6 +364,7 @@ void sw_map_redraw_rect(struct subwin *sw, SDL_Rect *area, struct band *band, in
     
 	if (sw->showwwls) plot_wkd_wwls(sw, sw->l1map, area);
     plot_cor(sw, sw->l1map, area, flags);
+	//z_dumpbitmap(NULL, sw->l1map);
 	//plot_qth(sw, area);
 //        dumpbitmap(sw->l1map);
     if (ctest && qrv && !fp) {
@@ -338,6 +380,10 @@ void sw_map_redraw_rect(struct subwin *sw, SDL_Rect *area, struct band *band, in
 //  dbg("                    %dx%d%+d%+d\n", area->x, area->y, area->w, area->h);
   
     sw_map_redraw_rotar(sw, area, flags);
+	if (band && band->tmplocqso.locator && *band->tmplocqso.locator){
+		plot_qso(sw, sw->screen, band, &band->tmplocqso);
+		if (gacs) plot_path(sw, sw->screen, band, &band->tmplocqso, &gacs->tmpqso);
+	}
 	plot_qth(sw, area);
 }					  
 
@@ -407,6 +453,7 @@ void sw_map_redraw_rain(struct subwin *sw, SDL_Rect *aarea, int flags){
 		if (grain->wetteronline != NULL) plot_rain_provider(sw, aarea, grain->wetteronline, ci);
 		if (grain->meteox != NULL) plot_rain_provider(sw, aarea, grain->meteox, ci);
 		if (grain->chmi != NULL) plot_rain_provider(sw, aarea, grain->chmi, ci);
+		if (grain->rainviewer != NULL) plot_rain_provider(sw, aarea, grain->rainviewer, ci);
 		if (grain->debug) break; 
 	} 
 
@@ -451,10 +498,12 @@ void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider
 		for (iy = 0; iy < provider->img->h; iy++){
 			for (ix = 0; ix < provider->img->w; ix++){
 				c = z_getpixel_fmt(provider->img, ix, iy, sdl->screen->format);
-				
-				provider->img2hw(provider, ix, iy, &hh1, &ww1);
+				//if (c == 0x00aad3df) continue;
 
-				hw2km(sw->myh, sw->myw, hh1, ww1, &kx, &ky);
+				
+				provider->img2hw(provider, provider->rv_px0 + ix, provider->rv_py0 + iy, &hh1, &ww1);
+
+				hw2km(gses->myh, gses->myw, hh1, ww1, &kx, &ky);
 				km2px(sw, kx, ky, &px1, &py1);
         
 				if (!z_overlapped_pixel(aarea, px1, py1)) continue;
@@ -508,10 +557,10 @@ void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider
 				hh1 = provider->h1 + (provider->left - provider->hpx1) * provider->hmult;
 				hh2 = provider->h1 + (provider->right + 1 - provider->hpx1) * provider->hmult;
 
-				hw2km(sw->myh, sw->myw, hh1, ww1, &kx, &ky);
+				hw2km(gses->myh, gses->myw, hh1, ww1, &kx, &ky);
 				km2px(sw, kx, ky, &px1, &py1);
 
-				hw2km(sw->myh, sw->myw, hh2, ww1, &kx, &ky);
+				hw2km(gses->myh, gses->myw, hh2, ww1, &kx, &ky);
 				km2px(sw, kx, ky, &px2, &py2);
 
 				if (iy > provider->top){
@@ -530,10 +579,10 @@ void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider
 				ww1 = provider->w1 + (provider->top - provider->wpx1) * provider->wmult;
 				ww2 = provider->w1 + (provider->bottom + 1 - provider->wpx1) * provider->wmult;
 
-				hw2km(sw->myh, sw->myw, hh1, ww1, &kx, &ky);
+				hw2km(gses->myh, gses->myw, hh1, ww1, &kx, &ky);
 				km2px(sw, kx, ky, &px1, &py1);
 
-				hw2km(sw->myh, sw->myw, hh1, ww2, &kx, &ky);
+				hw2km(gses->myh, gses->myw, hh1, ww2, &kx, &ky);
 				km2px(sw, kx, ky, &px2, &py2);
 
 				if (ix > provider->left){
@@ -571,7 +620,8 @@ void plot_rain_titles(SDL_Surface *surface, SDL_Rect *area, int flags){
     plot_rain_title(surface, area, flags, grain->chmi, &y);
     plot_rain_title(surface, area, flags, grain->wetteronline, &y);
     plot_rain_title(surface, area, flags, grain->meteox, &y);
-    plot_rain_title(surface, area, flags, grain->weatheronline, &y);
+	plot_rain_title(surface, area, flags, grain->weatheronline, &y);
+	plot_rain_title(surface, area, flags, grain->rainviewer, &y);
 }
 
 void sw_map_redraw(struct subwin *sw, struct band *band, int flags){
@@ -971,7 +1021,7 @@ void plot_qso(struct subwin *sw, SDL_Surface *surface, struct band *band, struct
 /*       dbg("kreslim qso\n"); */
     color=qso==sw->minqso?sdl->red:sdl->yellow2;
     if (qso==&band->tmplocqso) {
-		color=sdl->green;
+		color=sdl->magenta;
 	}
     z_cross(surface, px, py, color, sw->zoom);
 }
@@ -994,10 +1044,6 @@ void plot_qsos(struct subwin *sw, SDL_Surface *surface, struct band *band, SDL_R
         if (qso->error) continue;
         plot_qso(sw, surface, band, qso);
     }
-    if (band->tmplocqso.locator && *band->tmplocqso.locator){
-        plot_qso(sw, surface, band, &band->tmplocqso);
-		if (gacs) plot_path(sw, surface, band, &band->tmplocqso, &gacs->tmpqso);
-	}
 }
 
 void plot_rotars(struct subwin *sw, SDL_Surface *surface, SDL_Rect *area, SDL_Rect *outarea){
@@ -1103,13 +1149,10 @@ int map_update_qth(struct subwin *sw){
     sw->ox=sw->map.w/2; 
     sw->oy=sw->map.h/2; 
 	
-    sw->myh = qth(sw->pwwlo, 0);
-	sw->myw = qth(sw->pwwlo, 1);
- //   dbg("   %s=%d %d\n", sw->pwwlo, sw->myh, sw->myw);
     
-    zcors_recalc(sw->myh, sw->myw, sw->pwwlo, zsel, map_cor_callback, cor_tucnakcor, COR_ITEMS);
+	zcors_recalc(gses->myh, gses->myw, sw->pwwlo, zsel, map_cor_callback, cor_tucnakcor, COR_ITEMS);
 
-	acs_update_qth(gacs, sw);
+	acs_update_qth(gacs);
 
     return 1;
 }
@@ -1132,19 +1175,8 @@ int maps_reload(){
 
     }
     
-/*    invalidate_bkg(NULL);   in map_update_qth
-    invalidate_cache(aband);
-    clear_gfxstats();
-    gfx->dirty=1;
-    gfx->draw=1;*/
-    if (gses && gses->ontop && gses->ontop->type==SWT_MAP) {
-		gses->ontop->gdirty=1;
-		qrv_recalc_wkd(qrv);
-        qrv_recalc_qrbqtf(qrv);
-        qrv_recalc_gst(qrv);
-        map_recalc_gst(gses->ontop, aband);
-		sw_map_redraw(gses->ontop, aband, 0);
-	}
+	// next in map_cor_callback
+    
     return 0;
 }
 
@@ -1199,7 +1231,8 @@ void map_recalc_cors(){
     for (i=0;i<gses->subwins->len;i++){
         sw=(struct subwin *)g_ptr_array_index(gses->subwins, i);
         if (sw->type!=SWT_MAP) continue;
-        zcors_recalc(sw->myh, sw->myw, wwl, zsel, map_cor_callback, cor_tucnakcor, COR_ITEMS);
+		//zcors_recalc(sw->myh, sw->myw, wwl, zsel, map_cor_callback, cor_tucnakcor, COR_ITEMS);
+		zcors_recalc(gses->myh, gses->myw, wwl, NULL, NULL, cor_tucnakcor, COR_ITEMS);
         break; /* more maps with different pwwlo. but why? */
     }
 } 
@@ -1423,7 +1456,7 @@ int plot_qrb_qth(struct subwin *sw, SDL_Surface *surface, int mouse_x, int mouse
     zsdl_printf(surface, x, y, sdl->green, 0, ZFONT_TRANSP, VTEXT(T_GQTF), (int)(qtf*180/M_PI)); y+=FONT_H+4;
    /* fontoutf(surface, x, y, sdl->green, FONT_TRANSP, VTEXT(T_GINC), (int) (gst[(int)(qtf*180/M_PI)%360]/400*100));y+=FONT_H+4; */
 #if -1
-    if (qrbqtf2hw(sw->myh, sw->myw, qrb, qtf, &h2, &w2)==0){
+	if (qrbqtf2hw(gses->myh, gses->myw, qrb, qtf, &h2, &w2) == 0){
         zsdl_printf(surface, x, y, sdl->green, 0, ZFONT_TRANSP, VTEXT(T_GLON), h2*180.0/M_PI, x2gramin(buf, sizeof(buf), h2*180/M_PI, "EW")); y+=FONT_H+4;
         zsdl_printf(surface, x, y, sdl->green, 0, ZFONT_TRANSP, VTEXT(T_GLAT), w2*180.0/M_PI, x2gramin(buf, sizeof(buf), w2*180/M_PI, "NS")); y+=FONT_H+4;
         zsdl_printf(surface, x, y, sdl->green, 0, ZFONT_TRANSP, VTEXT(T_GLOC), hw2loc(buf, h2*180/M_PI, w2*180/M_PI, 6)); y+=FONT_H+4;
@@ -1627,9 +1660,14 @@ void plot_hgts(struct subwin *sw, SDL_Rect *area){
 void map_cor_callback(void)
 {
 	if (!gses) return;
-    if (gses->ontop->type!=SWT_MAP) return;
-    gses->ontop->gdirty = 1;
-    sw_map_redraw(gses->ontop, aband, 0);
+	if (gses && gses->ontop && gses->ontop->type == SWT_MAP) {
+		gses->ontop->gdirty = 1;
+		qrv_recalc_wkd(qrv);
+		qrv_recalc_qrbqtf(qrv);
+		qrv_recalc_gst(qrv);
+		map_recalc_gst(gses->ontop, aband);
+		sw_map_redraw(gses->ontop, aband, 0);
+	}
 }
 
 #else
