@@ -15,6 +15,7 @@
 #include "charsets.h"
 #include "chart.h"
 #include "control.h"
+#include "zcor.h"
 #include "cordata.h"
 #include "cwdaemon.h"
 #include "cwdb.h"
@@ -293,9 +294,9 @@ static void really_close_ctest(void)
 	map_recalc_cors();
 	maps_reload();
 	maps_update_showwwls();
+	rain_reload();
 #endif   
 	chart_reload();
-	rain_reload();
 }
 
 void menu_save_all(void *arg){
@@ -953,7 +954,7 @@ void menu_confirm_phase(void *xxx, char *phase){
 }
 
 
-void menu_phase(){
+void menu_phase(void){
 	g_snprintf(phase_str, EQSO_LEN, "%d", ctest->phase);
 	input_field(NULL, VTEXT(T_PHASE), VTEXT(T_ENTER_PHASE),
 		VTEXT(T_OK), VTEXT(T_CANCEL), gses,
