@@ -947,7 +947,7 @@ void wizz_func (void *arg){
     total_method=wi->total_method;
     safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method)),MAX_STR_LEN);
     /*safe_strncpy0(total_method_str,total_method_msg[total_method],MAX_STR_LEN);*/
-    log_addf("qsop_method=%d  total_method=%d", qsop_method, total_method);
+    //log_addf("qsop_method=%d  total_method=%d", qsop_method, total_method);
     /* wwltype */
     /* minqsop, maxqsop */
    
@@ -1036,7 +1036,7 @@ void refresh_contest_options1(void *arg)
         g_free(descfile);
     }
 
-    updloc=strcmp(cfg->pwwlo, pwwlo);
+	updloc = 1;// strcasecmp(cfg->pwwlo, pwwlo);
     
     STORE_STR_FS_UC(cfg,pcall); 
     STORE_STR_FS_UC(cfg,pclub);
@@ -1142,9 +1142,9 @@ void init_ctest2(int from_dlg, int updloc, struct zstring *zs2, struct wizz_item
             map_recalc_cors();
             maps_reload();
 			maps_update_showwwls();
+			rain_reload();
 #endif   
             chart_reload();
-			rain_reload();
 			//qrv_recalc_wkd(qrvdb); probably not needed
         }
         save_all_bands_txt(0);
@@ -1754,7 +1754,10 @@ void refresh_contest_def(void *xxx)
 	update_hw();
 
 #ifdef Z_HAVE_SDL
-    if (updloc) maps_reload();
+	if (updloc) {
+		update_hw();
+		maps_reload();
+	}
 	maps_update_showwwls();
 	rain_reload();
 
