@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2021  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -864,7 +864,7 @@ void menu_rig_opts(void *arg)
     d->items[i].fn   = check_number;
     d->items[i].gid  = 0;
     d->items[i].msg  = CTEXT(T_RIG_MODEL);
-    d->items[i].gnum = 20000;
+    d->items[i].gnum = 999999;
 
 	rig_model = atoi(rig_model_str);
     d->items[++i].type = D_BUTTON;   /* 2 */
@@ -1099,7 +1099,7 @@ char rot_serial[EQSO_LEN];
 char rot_timeout_ms_str[EQSO_LEN];
 char rot_poll_ms_str[EQSO_LEN];
 char rot_beamwidth_str[EQSO_LEN];
-char rot_saddr_str[EQSO_LEN], rot_model_str[EQSO_LEN];
+char rot_saddr_str[EQSO_LEN], rot_model_str[EQSO_LEN], rot_speed_str[EQSO_LEN];
 char rot_rem_rotstr[EQSO_LEN];
 
 void refresh_rotar_opts(void *xxx){
@@ -1122,7 +1122,8 @@ void refresh_rotar_opts(void *xxx){
     STORE_SINT(crot, rot_timeout_ms);
     STORE_SINT(crot, rot_poll_ms);
     STORE_SINT(crot, rot_beamwidth);
-    STORE_SINT(crot, rot_model);
+	STORE_SINT(crot, rot_model);
+	STORE_SINT(crot, rot_speed);
     STORE_SINT(crot, rot_saddr);
     STORE_STR (crot, rot_rem_rotstr); 
 
@@ -1157,7 +1158,8 @@ void menu_rotar_opts(void *arg){
         strcpy(rot_poll_ms_str,"500");
         strcpy(rot_beamwidth_str, "20");
         strcpy(rot_saddr_str, "240");
-        strcpy(rot_model_str, "0");
+		strcpy(rot_model_str, "0");
+		strcpy(rot_speed_str, "0");
         strcpy(rot_rem_rotstr, "A");
     }else{
         crot = (struct config_rotar *)g_ptr_array_index(cfg->crotars, roti);
@@ -1174,12 +1176,13 @@ void menu_rotar_opts(void *arg){
         LOAD_SINT (crot, rot_poll_ms);
         LOAD_SINT (crot, rot_beamwidth);
         LOAD_SINT (crot, rot_saddr);
-        LOAD_SINT (crot, rot_model);
+		LOAD_SINT (crot, rot_model);
+		LOAD_SINT (crot, rot_speed);
         LOAD_STR  (crot, rot_rem_rotstr); 
     }
 
-    if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
-    memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
+    if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 30 * sizeof(struct dialog_item)))) return;
+    memset(d, 0, sizeof(struct dialog) + 30 * sizeof(struct dialog_item));
     d->title = VTEXT(T_ROTAROPTS);
     d->fn = dlg_pf_fn;
     d->refresh = (void (*)(void *))refresh_rotar_opts;
@@ -1326,7 +1329,17 @@ void menu_rotar_opts(void *arg){
     d->items[i].msg = CTEXT(T_MODEL);
     d->items[i].wrap = 1;
     
-    d->items[++i].type = D_FIELD;
+	d->items[++i].type = D_FIELD;
+	d->items[i].dlen = EQSO_LEN;
+	d->items[i].data = rot_speed_str;
+	d->items[i].fn = check_number;
+	d->items[i].gid = 0;
+	d->items[i].gnum = 9999999;
+	d->items[i].maxl = 8;
+	d->items[i].msg = TRANSLATE("Hamlib speed:");
+	d->items[i].wrap = 1;
+
+	d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = rot_rem_rotstr;
     d->items[i].maxl = 3;
