@@ -15,6 +15,7 @@
 #include "dsp.h"
 #include "httpd.h"
 #include "inputln.h"
+#include "kbd.h"
 #include "main.h"
 #include "menu.h"
 #include "net.h"
@@ -325,7 +326,7 @@ void menu_set_mode(void *itdata, void *menudata)
 }
 
 
-//---- A/C --------------------------------------------------------------------
+//---- AS --------------------------------------------------------------------
 
 int ac_enable;       
 char ac_url[MAX_STR_LEN];        
@@ -353,6 +354,7 @@ void refresh_ac_opts(void *arg){
 
 	free_acs(gacs);
 	gacs = init_acs();
+	resize_terminal(NULL);
 	progress(NULL);
 }
 
