@@ -43,7 +43,7 @@ void free_ntpq(struct ntpq *ntpq){
 void ntpq_timer(void *arg){
     int fds[2], i;
 	struct ntpq *ntpq;
-    char errbuf[1024];
+    //char errbuf[1024];
 	
     /*dbg("ntpq_timer()\n");*/
 	ntpq=(struct ntpq *)arg;
@@ -67,7 +67,9 @@ void ntpq_timer(void *arg){
 		if (dup(fds[1]) < 0) exit(-1);
 		if (dup(fds[1]) < 0) exit(-1);
         execlp("ntpq", "ntpq", "-pn", NULL);
-        fprintf(stderr, " exec failed errno=%d %s\n", errno, strerror_r(errno, errbuf, sizeof(errbuf)));
+        char *c = z_strdup_strerror(errno);
+        fprintf(stderr, " exec failed errno=%d %s\n", errno, c);
+        g_free(c);
         exit(-1);
     }                  
     /* parent */
