@@ -397,8 +397,8 @@ struct band *init_band(struct config_band *confb, GHashTable *opt_band, struct z
         inputln_func(b->il,&ev);
     }
 
-    b->stats = init_stats();
-    b->tmpstats = init_stats();
+    b->stats = init_stats(b->bandchar);
+	b->tmpstats = init_stats(b->bandchar);
     MUTEX_INIT(b->stats);
     
     c = g_strdup_printf("%s/%c.swp",ctest->directory, z_char_lc(b->bandchar));
@@ -878,7 +878,9 @@ void activate_band(struct band *b){
     redraw_later();
     maps_reload();
     chart_reload();
+#ifdef Z_HAVE_SDL
 	rain_reload();
+#endif
 }
 
 
@@ -1377,10 +1379,13 @@ void free_ctest(){
     
     set_ctest_title();
     cq_abort(1);
+	update_hw();
     if (!zselect_terminating(zsel))	maps_reload();
 	maps_update_showwwls();
     chart_reload();
+#ifdef Z_HAVE_SDL
 	rain_reload();
+#endif
 }
 
 
