@@ -26,6 +26,8 @@
 
 struct rain *grain;
 
+#define MAXLAT (85.05112878 * M_PI / 180.0)
+
 struct rain *init_rain(){
     struct rain *rain;
     int i;
@@ -208,6 +210,24 @@ struct rain *init_rain(){
 
 		rain->weatheronline = provider;
 	}
+
+	if (cfg->rain_rainviewer){
+		struct rain_provider *provider = g_new0(struct rain_provider, 1);
+		dbg("new rainviewer %p\n", provider);
+		provider->rain = rain;
+		provider->name = "rainviewer";
+		provider->pal = rain->bwcolors;
+		g_assert(provider->timer_id == 0);
+		provider->timer_id = zselect_timer_new(zsel, 1000, rain_rainviewer_timer, provider);
+		rain_rainviewer_init(provider);
+
+		provider->hash_func = rain_rainviewer_hash;
+		provider->img2hw = rain_rainviewer_img2hw;
+		for (i = 0; i < RAIN_COLORS; i++) provider->hashes[i] = g_hash_table_new(g_direct_hash, g_direct_equal);
+		provider->mask = 0xffffff80;
+		provider->step = 0x80;
+		rain->rainviewer = provider;
+	}
 	
     return rain;
 }
@@ -251,9 +271,10 @@ void rain_analyze_scps(struct rain *rain){
 	if (rain->meteox) rain_add_scps(rain->meteox);
 	if (rain->wetteronline) rain_add_scps(rain->wetteronline);
 	if (rain->weatheronline) rain_add_scps(rain->weatheronline);
+	if (rain->rainviewer) rain_add_scps(rain->rainviewer);
 	
 	if (rain->scplocs->len > 0){
-		log_addf("Scatterpoints: %s", rain->scplocs->str);
+		log_addf(TRANSLATE("Scatterpoints: %s"), rain->scplocs->str);
 	}
 }
 
@@ -333,7 +354,11 @@ void free_rain_provider(struct rain_provider *provider){
 	g_free(provider->srccols);
 	g_free(provider->translation);
 	g_free(provider->title);
+	g_free(provider->rv_path);
+	g_free(provider->rv_oldpath);
 	g_free(provider);
+	dbg("freed rainviewer %p\n", provider);
+
 }
 
 void free_rain(struct rain *rain){
@@ -343,10 +368,11 @@ void free_rain(struct rain *rain){
 	if (rain->wetteronline) free_rain_provider(rain->wetteronline);
 	if (rain->chmi) free_rain_provider(rain->chmi);
 	if (rain->weatheronline) free_rain_provider(rain->weatheronline);
+	if (rain->rainviewer) free_rain_provider(rain->rainviewer);
 
     g_free(rain->colors);
     g_free(rain->bwcolors);
-	g_string_free(rain->scplocs, TRUE);
+    g_string_free(rain->scplocs, TRUE);
     g_free(rain);
 }
 
@@ -363,8 +389,8 @@ static void rain_provider_add(struct rain_provider *provider, int ix, int iy, in
 	struct zbinbuf *zbb;
 	struct rain_km km;
 
-    provider->img2hw(provider, ix, iy, &hh1, &ww1); 
-    provider->img2hw(provider, ix + 2, iy + 2, &hh2, &ww2);// when +1 space between bars 
+    provider->img2hw(provider, ix - 1, iy - 1, &hh1, &ww1); 
+    provider->img2hw(provider, ix + 1, iy + 1, &hh2, &ww2);// when +1 space between bars 
 
 	//hh2 = hh1 + provider->hmult;
 	//ww2 = ww1 + provider->wmult;
@@ -418,7 +444,7 @@ void rain_provider_draw_colors(struct rain_provider *provider, SDL_Surface *surf
 	}
 }
 
-void rain_linear_img2hw(struct rain_provider *provider, int ix, int iy, double *h, double *w){
+void rain_linear_img2hw(struct rain_provider *provider, float ix, float iy, double *h, double *w){
 	*h = provider->h1 + (ix - provider->hpx1) * provider->hmult;
 	*w = provider->w1 + (iy - provider->wpx1) * provider->wmult;
 }
@@ -508,7 +534,9 @@ void rain_meteox_downloaded(struct zhttp *http){
 	}
 	
 
-	if (provider->ofs_min >= -60 && strncmp(http->response->buf + http->dataofs, "GIF89a", 6) != 0){ // image not available, go to the past
+	if (provider->ofs_min >= -60 && 
+		(http->dataofs < 1 || strncmp(http->response->buf + http->dataofs, "\x89PNG\r\n\x1a\n", 8) != 0) // PNG is OK, GIF is error
+       ){ // image not available, go to the past
 		provider->ofs_min -= 15;
 		dbg("set meteox_ofs_min to %d\n", provider->ofs_min);
 		zhttp_free(http);
@@ -884,7 +912,7 @@ void rain_chmi_load(struct rain_provider *provider){
 			for (ci = 0; ci < RAIN_COLORS; ci++){
 				if (c == provider->rain->colors[ci]){
 					if (ci < 5) break;
-					rain_provider_add(provider, ix, iy, ci);
+					rain_provider_add(provider, ix, iy, ci/*9*/);
 					break;
 				}
 			}
@@ -1080,7 +1108,7 @@ gpointer rain_weatheronline_hash(short kx, short ky, int ci){
 }
 
 
-void rain_weatheronline_img2hw(struct rain_provider *provider, int ix, int iy, double *h, double *w){
+void rain_weatheronline_img2hw(struct rain_provider *provider, float ix, float iy, double *h, double *w){
 	double dx = ix - 234;
 	double dy = iy + 280;
 	double d = sqrt(dx * dx + dy * dy) / 12.6 - 4.75;
@@ -1094,7 +1122,88 @@ void rain_weatheronline_img2hw(struct rain_provider *provider, int ix, int iy, d
 }
 
 /* www.rainviewer.com */
-int tile_googlezoom(int size, double dh, double dw, int px, int py)
+void rain_rainviewer_init(struct rain_provider *provider){
+	if (gses == NULL || provider == NULL) return;
+
+	provider->rv_size = 256;
+	double qrb = cfg->rain_maxqrb; // TODO
+	double qtf;
+	double maxh = 0, maxw = 0;
+	for (qtf = 0.0; qtf <= 180.0; qtf += 30.0){
+		double h, w;
+		qrbqtf2hw(/*gses->myh*/ 0, gses->myw, qrb, qtf * M_PI / 180.0, &h, &w);
+		double dw = fabs(w - gses->myw);
+		if (h > maxh) maxh = h;
+		if (dw > maxw) maxw = dw;
+		//dbg("h=%3.1f  w=%3.1f\n", h * 180.0 / M_PI, w * 180 / M_PI);
+	}
+	dbg("qrb=%1.0f  maxh=%3.1f  maxw=%3.1f\n", qrb, maxh * 180.0 / M_PI, maxw * 180 / M_PI);
+
+	provider->rv_zoom = tile_googlezoom(2 * maxh, 2 * maxw);
+	provider->rv_pow = pow(2, provider->rv_zoom);
+	provider->rv_mul = provider->rv_size * provider->rv_pow;
+	dbg("zoom=%d  pow=%d  mul=%d\n", provider->rv_zoom, provider->rv_pow, provider->rv_mul);
+	//provider->rv_zoom = 0;
+
+	rain_rainviewer_hw2himg(provider, gses->myh, gses->myw, &provider->rv_mypx, &provider->rv_mypy);
+	dbg("mypx=%d  mypy=%d\n", provider->rv_mypx, provider->rv_mypy);
+	provider->rv_px0 = provider->rv_mypx - (provider->rv_size -1) / 2.0;
+	provider->rv_py0 = provider->rv_mypy - (provider->rv_size - 1) / 2.0;
+	dbg("px0=%d  py0=%d\n", provider->rv_px0, provider->rv_py0);
+
+
+	double h, w;
+	rain_rainviewer_img2hw(provider, 0, 0, &h, &w);
+	dbg("0 0 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
+	rain_rainviewer_img2hw(provider, 128 * provider->rv_pow, 128 * provider->rv_pow, &h, &w);
+	dbg("128 128 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
+	rain_rainviewer_img2hw(provider, 256 * provider->rv_pow, 256 * provider->rv_pow, &h, &w);
+	dbg("256 256 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
+	dbg("\n");
+
+	/*rain_rainviewer_img2hw(provider, 4352, 2560, &h, &w);
+	dbg("4352 2560 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
+	rain_rainviewer_img2hw(provider, 4608, 2816, &h, &w);
+	dbg("4608 2816 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
+
+	int ix, iy;
+	rain_rainviewer_hw2himg(provider, 11.25 * M_PI / 180.0, 55.776575 * M_PI / 180.0, &ix, &iy);
+	dbg("%d %d\n", ix, iy);
+	rain_rainviewer_hw2himg(provider, 22.5 * M_PI / 180.0, 48.922497 * M_PI / 180.0, &ix, &iy);
+	dbg("%d %d\n", ix, iy);	   */
+
+
+
+	/*double h, w;
+	rain_rainviewer_img2hw(provider, gx, gy, &h, &w);
+	h *= 180 / M_PI;
+	w *= 180 / M_PI;*/
+
+	provider->hpx1 = provider->rv_px0;  // x
+	provider->hpx2 = provider->rv_px0 + provider->rv_size - 1; // x
+	provider->wpx1 = provider->rv_py0 + provider->rv_size - 1; // y
+	provider->wpx2 = provider->rv_py0; // y
+
+	provider->left = provider->rv_px0;
+	provider->top = provider->rv_py0;
+	provider->right = provider->rv_px0 + provider->rv_size - 1;
+	provider->bottom = provider->rv_py0 + provider->rv_size - 1;
+
+
+	rain_rainviewer_img2hw(provider, provider->hpx1, provider->wpx1+1, &provider->h1, &provider->w1);
+	rain_rainviewer_img2hw(provider, provider->hpx2+1, provider->wpx2, &provider->h2, &provider->w2);
+	dbg("dh = %3.2f,  dw = %3.2f \n", fabs(provider->h2 - provider->h1) * 180.0 / M_PI, fabs(provider->w2 - provider->w1) * 180.0 / M_PI);
+	dbg("h = %3.2f ... %3.2f   (my %3.2f)\n", provider->h1 * 180.0 / M_PI, provider->h2 * 180.0 / M_PI, gses->myh * 180.0 / M_PI);
+	dbg("w = %3.2f ... %3.2f   (my %3.2f) \n", provider->w1 * 180.0 / M_PI, provider->w2 * 180.0 / M_PI, gses->myw * 180.0 / M_PI);
+
+	provider->hmult = (provider->h2 - provider->h1) / (provider->hpx2 - provider->hpx1);
+	provider->wmult = (provider->w2 - provider->w1) / (provider->wpx2 - provider->wpx1);
+
+
+
+}
+
+int tile_googlezoom(double dh, double dw)
 {
 	double hexp, wexp, gzh, gzw;
 	int gz;
@@ -1104,30 +1213,213 @@ int tile_googlezoom(int size, double dh, double dw, int px, int py)
 		gzh = 0;
 	else
 		gzh = log(hexp) / log(2);
-	dbg("hexp=%4.2f  gzh=%4.2f \n", hexp, gzh);
+	//dbg("hexp=%4.2f  gzh=%4.2f \n", hexp, gzh);
 
-	wexp = M_PI / fabs(dw);
+	//wexp = M_PI / fabs(dw);
+	wexp = ((2 * MAXLAT) * M_PI) / fabs(dw);
 	if (wexp < 2.0)
 		gzw = 0;
 	else
 		gzw = log(wexp) / log(2);
-	dbg("wexp=%4.2f  gzw=%4.2f \n", wexp, gzw);
+	//dbg("wexp=%4.2f  gzw=%4.2f \n", wexp, gzw);
 
-	gz = (int)(Z_MAX(gzw, gzh));
-	dbg("gz=%d\n", gz);
+	gz = (int)floor(Z_MIN(gzw, gzh));
+	//dbg("gz=%d\n", gz);
+	if (gz > 20) gz = 20;
 
 	return (int)gz;
 }
 
-void tile_get_hwi(int gz, double h, double w, int *hi, int *wi){
+void rain_rainviewer_timer(void *arg){
+	char *url;
+	struct rain_provider *provider = (struct rain_provider*)arg;
 
-	if (h < 0.0 || h >= 360) *hi = -1;
-	if (w < 0.0 || w >= 180) *hi = -1;
+	provider->timer_id = 0;
+	
+	url = "https://api.rainviewer.com/public/weather-maps.json";
 
-	*hi = (int)((h + 180.0) * (1 << gz) / 360.0);
-	*wi = (int)((90.0 - w) * (1 << gz) / 360.0);
+	provider->http = zhttp_init();
+	dbg("rainviewer %p url = '%s'\n", provider, url);
+	zhttp_get(provider->http, zsel, url, rain_rainviewer_downloaded_json, provider);
+	g_assert(provider->timer_id == 0);
+	provider->timer_id = zselect_timer_new(zsel, 60 * 1000, rain_rainviewer_timer, provider);
 }
 
+void rain_rainviewer_downloaded_json(struct zhttp *http){
+	struct rain_provider *provider = (struct rain_provider*)http->arg;
+
+	
+	if (provider->timer_id) {
+		zselect_timer_kill(zsel, provider->timer_id);
+		provider->timer_id = 0;
+	}
+
+	if (http->state == ZHTTPST_ERROR && http->errorstr){
+		dbg("ERROR rain_rainviewer_downloaded_json: %s\n", http->errorstr);
+	}
+
+	dbg("rain_rainviewer_downloaded_json: got %d bytes\n", http->response->len - http->dataofs);
+
+	if (provider->rain->debug) zbinbuf_write_to_file(http->response, "_rainviewer.json", http->dataofs, http->response->len - http->dataofs);
+
+	char *json = g_new(char, http->response->len + 1);
+	zbinbuf_getstr(http->response, http->dataofs, json, http->response->len + 1);
+	zhttp_free(http);
+	provider->http = NULL;
+
+	rain_rainviewer_process_json(provider, json);
+	g_free(json);
+
+	g_assert(provider->timer_id == 0);
+	provider->timer_id = zselect_timer_new(zsel, 60 * 1000, rain_rainviewer_timer, provider);
+
+}
+
+void rain_rainviewer_process_json(struct rain_provider *provider, const char *json){
+	int len = strlen(json);
+	int i;
+	time_t tt = 0;
+	for (i = 0;; i++){
+		char key[246];
+		g_snprintf(key, sizeof(key), "radar.past.%d.time", i);
+		char *s = zjson_get_str(json, len, key);
+		if (s == NULL) break;
+
+		tt = (time_t)strtoll(s, NULL, 10);
+		//dbg("%d  tt=%lld\n", i, (long long)tt);
+
+		g_snprintf(key, sizeof(key), "radar.past.%d.path", i);
+		s = zjson_get_str(json, len, key);
+		if (s == NULL) break;
+
+		g_free(provider->rv_path);
+		provider->rv_path = s; 
+	}
+	char *host = zjson_get_str(json, len, "host");
+	struct tm tm;
+	gmtime_r(&tt, &tm);
+	//dbg("rainviewer latest = %lld = %02d:%02dZ\n", (long long)tt,  tm.tm_hour, tm.tm_min);
+
+	if (host != NULL && (provider->rv_oldpath == NULL || strcmp(provider->rv_path, provider->rv_oldpath) != 0)){
+		
+		g_free(provider->title);
+		provider->title = g_strdup_printf("RainV. %02d:%02dZ", tm.tm_hour, tm.tm_min);
+
+
+		char *url = g_strdup_printf("%s%s/%d/%d/%6.4f/%6.4f/0/0_0.png", host, provider->rv_path, provider->rv_size, provider->rv_zoom, gses->myw * 180.0 / M_PI, gses->myh * 180.0 / M_PI);
+		//url = g_strdup("https://tile.openstreetmap.org/0/0/0.png");
+		//url = g_strdup("https://tile.openstreetmap.org/5/17/10.png");
+
+		provider->http = zhttp_init();
+		dbg("rainviewer %02d:%02dZ = '%s'\n", tm.tm_hour, tm.tm_min, url);
+		zhttp_get(provider->http, zsel, url, rain_rainviewer_downloaded_image, provider);
+		g_free(url);
+	}
+
+	g_free(provider->rv_oldpath);
+	provider->rv_oldpath = g_strdup(provider->rv_path);
+
+}
+
+void rain_rainviewer_downloaded_image(struct zhttp *http){
+	struct rain_provider *provider = (struct rain_provider*)http->arg;
+	SDL_Surface *newimg;
+	
+	if (provider->timer_id) {
+		zselect_timer_kill(zsel, provider->timer_id);
+		provider->timer_id = 0;
+	}
+
+	if (http->state == ZHTTPST_ERROR && http->errorstr){
+		dbg("ERROR rain_rainviewer_downloaded_image: %s\n", http->errorstr);
+		zhttp_free(http);
+		provider->http = NULL;
+		g_assert(provider->timer_id == 0);
+		provider->timer_id = zselect_timer_new(zsel, 60 * 1000, rain_rainviewer_timer, provider);
+		g_free(provider->rv_oldpath);
+		provider->rv_oldpath = NULL;
+		return;
+	}
+
+
+	dbg("rain_rainviewer_downloaded_image: got %d bytes\n", http->response->len - http->dataofs);
+
+	if (provider->rain->debug) zbinbuf_write_to_file(http->response, "_rainviewer.png", http->dataofs, http->response->len - http->dataofs);
+
+
+	newimg = zpng_create(http->response->buf + http->dataofs, http->response->len - http->dataofs);
+	if (newimg == NULL){
+		dbg("Can't read rainviewer bitmap\n");
+		provider->pal = provider->rain->bwcolors;
+		g_free(provider->rv_oldpath);
+		provider->rv_oldpath = NULL;
+	}
+	else{
+		dbg("rainviewer bitmap loaded, %dx%d\n", newimg->w, newimg->h);
+		if (provider->img != NULL) {
+			SDL_FreeSurface(provider->img);
+		}
+		provider->img = newimg;
+		provider->pal = provider->rain->colors;
+		rain_rainviewer_load(provider);
+		rain_analyze_scps(provider->rain);
+
+	}
+	zhttp_free(http);
+	provider->http = NULL;
+	sw_set_gdirty(SWT_MAP);
+	g_assert(provider->timer_id == 0);
+	provider->timer_id = zselect_timer_new(zsel, 1 * 60 * 1000, rain_rainviewer_timer, provider);
+
+}
+
+
+void rain_rainviewer_load(struct rain_provider *provider){
+	int ix, iy, c, ci, i;
+	SDL_Surface *img = provider->img;
+
+	provider->w = img->w;
+	provider->h = img->h;
+
+	for (i = 0; i < RAIN_COLORS; i++) {
+		GHashTable *hash = provider->hashes[i];
+		g_hash_table_foreach(hash, rain_truncate_hash, NULL);
+	}
+
+
+	for (iy = 0; iy < img->h; iy++){
+		for (ix = 0; ix < img->w; ix++){
+			c = z_getpixel_fmt(img, ix, iy, sdl->screen->format);
+
+			int red = z_r(img, c);
+			ci = (red - 36) / 4;
+			//if (ci < 0) continue;
+			if (ci < 5) continue;
+			if (ci >= RAIN_COLORS) ci = RAIN_COLORS - 1;
+
+			rain_provider_add(provider, provider->rv_px0 + ix, provider->rv_py0 + iy, ci/*5*/);
+		}
+	}
+}
+
+gpointer rain_rainviewer_hash(short kx, short ky, int ci){
+	unsigned int key = ((unsigned short)kx) >> 7;
+	key |= (((unsigned short)ky) >> 1) & 0xff00;
+	key |= ((unsigned char)ci) << 16;
+	return GUINT_TO_POINTER(key);
+}
+
+// y = (ln(tan(pi/4 + (x*pi/180)/2)))  * 90/pi for x = -85 .. 85
+
+void rain_rainviewer_hw2himg(struct rain_provider *provider, double h, double w, int *x, int *y){
+	*x = provider->rv_mul * (h + M_PI) / (2 * M_PI);
+	*y = provider->rv_mul * (M_PI - log(tan(M_PI_4 + w / 2))) / (2 * M_PI);
+}
+
+void rain_rainviewer_img2hw(struct rain_provider *provider, float x, float y, double *h, double *w){
+	*h = (x * 2 * M_PI / provider->rv_mul) - M_PI;
+	*w = 2 * (atan(exp(M_PI - y * 2 * M_PI / provider->rv_mul)) - M_PI_4);
+}
 
 
 
