@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2021  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@
 #include "subwin.h"
 #include "zstring.h"
 
+
 struct rotar *rota,*rotb;
 //struct hdkeyb *hdkeyb;
 
@@ -50,6 +51,9 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
     rot = g_new0(struct rotar, 1);
 	rot->poll_ms = crot->rot_poll_ms;
 	rot->last_req = ztimeout_init(rot->poll_ms);
+#ifdef HAVE_HAMLIB
+	rot->rot_speed = crot->rot_speed;
+#endif
     switch(crot->rot_type){
         case ROT_OK1ZIA_TTYS:
             rot->sdev=sd_open_ttys(crot->rot_saddr, crot->rot_filename, crot->rot_timeout_ms);
@@ -98,18 +102,22 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
 #ifdef Z_MSC_MINGW_CYGWIN
 				if (strchr(crot->rot_filename, '\\') == NULL && z_strcasestr(crot->rot_filename, "COM") != NULL){
 					char *c = g_strdup_printf("\\\\.\\%s", crot->rot_filename);
-					safe_strncpy(rot->rot->state.rotport.pathname, c, FILPATHLEN - 1);
+					safe_strncpy(rot->rot->state.rotport.pathname, c, HAMLIB_FILPATHLEN - 1);
 					g_free(c);
 				}
 				else
 #endif
 				{
-					safe_strncpy(rot->rot->state.rotport.pathname, crot->rot_filename, FILPATHLEN - 1);
+					safe_strncpy(rot->rot->state.rotport.pathname, crot->rot_filename, HAMLIB_FILPATHLEN - 1);
 				}
-				TRROT(0, "rot->rot->state.rotport.pathname='%s'", rot->rot->state.rotport.pathname);
+				if (rot->rot_speed > 0) rot->rot->state.rotport.parm.serial.rate = rot->rot_speed;
+
+				TRROT(0, "rotar %c pathname='%s' speed=%d", rotchar, rot->rot->state.rotport.pathname, rot->rot->state.rotport.parm.serial.rate);
 			}
 
             rot->hl_azim = -1;
+			rot->hl_elev = -90;
+			rot->hl_lastelev = 0;
             rot->hl_model = crot->rot_model;
 			rot->hl_thread = g_thread_try_new("hamlib rotator", rot_hamlib_main, (gpointer)rot, NULL); 
 			if (!rot->hl_thread) {
@@ -241,7 +249,7 @@ int rot_update_colors(){
 }
 #endif
 
-int rot_seek(struct rotar *rot, int uhel){
+int rot_seek(struct rotar *rot, int uhel, int elev){
     char *c;
     struct sconn_job *job;
 
@@ -249,8 +257,8 @@ int rot_seek(struct rotar *rot, int uhel){
 
     switch (rot->type){
         case ROT_REMOTE:
-            c = g_strdup_printf("RS %s;%s;%d;%s\n", 
-                    rot->netid, rot->rem_rotstr, uhel, "");
+            c = g_strdup_printf("RS %s;%s;%d;%d\n", 
+                    rot->netid, rot->rem_rotstr, uhel, elev);
             rel_write_all(c);
             g_free(c);
             
@@ -258,6 +266,7 @@ int rot_seek(struct rotar *rot, int uhel){
 #ifdef HAVE_HAMLIB            
         case ROT_HAMLIB:
             rot->hl_azim = uhel;      
+			rot->hl_elev = elev;
             break;
 #endif
         default:
@@ -265,6 +274,7 @@ int rot_seek(struct rotar *rot, int uhel){
             job->cmd = SCONN_ROT_AZIM;     
             job->sdev = rot->sdev;
             job->azim = uhel;    
+			job->elev = elev;
             sconn_job_add(rot->sdev->sconn, job);
             break;
     }
@@ -352,10 +362,12 @@ void rotar_read_handler(int n, char **items){
         log_addf(VTEXT(T_ROTAR_ERROR_S), items[3]);
     }
     if (strcasecmp(cmd, "QTF")==0){
-        char *qtf = items[3];
+		char *qtf = items[3];
+		char *elev = items[4];
         rot->qtf=atoi(qtf);
+		rot->elev = atoi(elev);
         
-        zs2 = zconcatesc(gnet->myid, rotstr, qtf, "0", NULL);
+        zs2 = zconcatesc(gnet->myid, rotstr, qtf, elev, NULL);
         c = g_strconcat("RU ", zs2->str, "\n", NULL);
         rel_write_all(c);
         g_free(c);
@@ -392,7 +404,7 @@ void timer_update_rotar(void *arg){
 int rotar_main(struct sdev *sdev){
     struct rotar *rot = (struct rotar *)sdev->arg;
 	char data[256];
-	int len, ret, qtf;
+	int len, ret, qtf, elev;
   //  static int xx = 0;
 
     //dbg("rotar_main rot=%p\n", rot);
@@ -408,15 +420,17 @@ int rotar_main(struct sdev *sdev){
     //dbg("sd_prot ret=%d\n", ret);
     if (ret != 0) return ret;
 
-    qtf = (unsigned char)data[0]+256*(unsigned char)data[1];
+	qtf = (unsigned char)data[0] + 256 * (unsigned char)data[1];
+	elev = (int16_t)((unsigned char)data[2] + 256 * (unsigned char)data[3]);
   //  qtf = (xx++ % 360);//FIXME
-	trace(cfg->trace_sdev, "qtf=%d old=%d\n", qtf, rot->oldqtf);
-    if (qtf == rot->oldqtf) return 0;
+	trace(cfg->trace_sdev, "qtf=%d old=%d  elev=%d old=%d\n", qtf, rot->oldqtf, elev, rot->oldelev);
+    if (qtf == rot->oldqtf && elev == rot->oldelev) return 0;
     
     rot->oldqtf = qtf;
+	rot->oldelev = elev;
     //g_snprintf(s, sizeof(s), "%d", qtf);
     //ret = zwrite(tpipe->threadpipe_write, zconcatesc("ROT", rot->rotstr, "QTF", s, NULL));
-	zselect_msg_send(zsel, "%s;%s;%s;%d", "ROT", rot->rotstr, "QTF", qtf);
+	zselect_msg_send(zsel, "%s;%s;%s;%d;%d", "ROT", rot->rotstr, "QTF", qtf, elev);
 //    dbg("zwrite ret=%d\n", ret);
     return 0;
 }
@@ -446,7 +460,13 @@ void rotar_remote_update(char *c){
 
     rot->qtf = atoi(qtf);
     rot->elev = atoi(elev);
-    redraw_later();
+	redraw_later();
+#ifdef Z_HAVE_SDL
+	if (gses->ontop->type == SWT_MAP){
+		gses->ontop->gdirty = 1;
+	}
+#endif
+
 x:;
     zfree(zs);
     
@@ -459,10 +479,11 @@ static char rotar_qtf_str[EQSO_LEN], rotar_elev_str[EQSO_LEN];
 static char rotar_desc1[MAX_STR_LEN], rotar_desc2[MAX_STR_LEN];
 
 void refresh_rotar(void *xxx){
-    if (strlen(rotar_qtf_str)>0){
-        rot_seek(rotar, atoi(rotar_qtf_str));
-    }
-    /*dbg("refresh_rotar\n");*/
+	if (strlen(rotar_qtf_str) <= 0) return;
+	int elev = -90;
+	if (strlen(rotar_elev_str) > 0) elev = atoi(rotar_elev_str);
+	ac_track(gacs, NULL, -10);
+	rot_seek(rotar, atoi(rotar_qtf_str), elev);
 }
 
 char *rotar_msg[] = {
@@ -623,7 +644,7 @@ gpointer rot_hamlib_main(gpointer xxx){
     struct rotar *rot = (struct rotar*)xxx;
     azimuth_t azimuth;
     elevation_t elevation;
-    int qtf;
+    int qtf, elev;
 	int tmo;
 
     dbg("rot_hl_main\n");
@@ -645,7 +666,8 @@ gpointer rot_hamlib_main(gpointer xxx){
         
         if (rot->hl_azim >= 0){
             azimuth_t az = rot->hl_azim;
-            elevation_t el = 0;
+            elevation_t el = rot->hl_elev;
+			if (el <= -90) el = rot->hl_lastelev;
 
             ret = rot_set_position(rot->rot, az, el);
 			TRROT(1, "rot_set_position(az %3.1f, el %3.1f) = %d", (double)az, (double)el, ret);
@@ -655,11 +677,14 @@ gpointer rot_hamlib_main(gpointer xxx){
 		ret = rot_get_position(rot->rot, &azimuth, &elevation);
 		if (ret == RIG_OK){
 			TRROT(1, "rot_get_position() = az %3.1f, el %3.1f", (double)azimuth, (double)elevation);
-			qtf = (int)azimuth;
-			if (qtf != rot->oldqtf){
+			rot->hl_lastelev = elevation;
+			qtf = (int)roundf((float)azimuth);
+			elev = (int)roundf((float)elevation);
+			if (qtf != rot->oldqtf || elev != rot->elev){
 				rot->oldqtf = qtf;
+				rot->oldelev = elev;
 				// g_snprintf(s, sizeof(s), "%d", qtf);
-				zselect_msg_send(zsel, "ROT;%s;QTF;%d", rot->rotstr, qtf);
+				zselect_msg_send(zsel, "ROT;%s;QTF;%d;%d", rot->rotstr, qtf, elev);
 				// ret = zwrite(tpipe->threadpipe_write, zconcatesc("ROT", rot->rotstr, "QTF", s, NULL));
 				//    dbg("zwrite ret=%d\n", ret);
 			}
