@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2011 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2011-2021 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -59,6 +59,7 @@ struct config_rotar{
 
     /* hamlib */
     gint rot_model;
+	gint rot_speed;
 
     /* remote */
     gchar *rot_rem_rotstr;
@@ -83,7 +84,7 @@ struct rotar{
     int verh, verl, identa;
     
     /* state */
-    int qtf, oldqtf, elev;
+    int qtf, oldqtf, elev, oldelev;
 //    int timer_id;           /* time between two command to prevent high cpu load */
 
     int color; /* color of arrow in polar map */
@@ -92,10 +93,12 @@ struct rotar{
     int last_req;
 #ifdef HAVE_HAMLIB
     GThread *hl_thread;
-    int hl_azim;
+    int hl_azim, hl_elev;
     int hl_model;
     int hl_give_me_chance;
     int hl_thread_break;
+	int rot_speed;
+	elevation_t hl_lastelev;
 #endif
 	int poll_ms;
 };
@@ -111,7 +114,7 @@ int free_rotars(void);
 
 struct rotar *init_rotar(struct config_rotar *cfg, int rotchar);
 void free_rotar(struct rotar *rot);
-int rot_seek(struct rotar *rot, int uhel);
+int rot_seek(struct rotar *rot, int uhel, int elev /* -90 unused */);
 struct config_rig *get_config_rig_by_number(GPtrArray *crigs, int nr);
 struct config_rotar *get_config_rotar_by_number(GPtrArray *rotars, int nr);
 struct rotar *get_rotar(int nr);
