@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2013-2018 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2013-2021 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -316,7 +316,7 @@ void free_sdr(struct sdr *sdr){
 }
 
 struct sdr *sdr_start(struct sdr *sdr){
-	dbg("sdr_start(%p)\n", sdr);
+	//dbg("sdr_start(%p)\n", sdr);
 
     if (!sdr) return sdr;
 	if (sdr->thread) return sdr;
