@@ -326,10 +326,6 @@ gpointer ssbd_play_thread_func(gpointer data){
         gssbd->playedf += frames;
 
         /* todo playmax & etc */
-#if 0
-//        dbg("ctrl=%d\n", sdl->ctrl);
-        if (!sdl->ctrl) memset(gssbd->buffer, 0, gssbd->bytes);
-#endif
 
         if (gssbd->proc_break)  {
             gssbd->dsp->reset(gssbd->dsp);
