@@ -27,6 +27,11 @@ char *get_wwl(char *buf, char *wwl){
     return buf;
 }
 
+//static void enum_valid (gpointer key, gpointer value, gpointer user_data){
+//    dbg("%s key='%p'\n", user_data, key);
+//    dbg("%s key='%s'\n", user_data, key);
+//    dbg("%s value='%p'\n", user_data, value);
+//}
 
 void draw_one_bigdigit(int x, int y, int num){
     switch(num%10){
@@ -99,10 +104,12 @@ void draw_bigdigit(int x, int y, int num){
     
 }
 
-struct stats *init_stats(){
+struct stats *init_stats(char bandchar){
     struct stats *st;
 
     st = (struct stats *)g_new0(struct stats, 1); 
+	st->bandchar = bandchar;
+	//dbg("new stats %p for %c\n", st, st->bandchar);
 
     st->wwls  = g_hash_table_new(g_str_hash, g_str_equal);
     st->dxcs  = g_hash_table_new(g_str_hash, g_str_equal);
@@ -120,7 +127,7 @@ struct stats *init_stats(){
 void free_stats(struct stats *st){
 
     clear_stats(st);
-    
+
     g_hash_table_destroy(st->wwls);
     g_hash_table_destroy(st->dxcs);
     g_hash_table_destroy(st->prefs);
@@ -154,7 +161,11 @@ void clear_stats(struct stats *st){
     g_hash_table_foreach_remove(st->dxcs,  free_gpointer_item, NULL);
     g_hash_table_foreach_remove(st->prefs, free_gpointer_item, NULL);
     g_hash_table_foreach_remove(st->excs,  free_gpointer_item, NULL);
+    //dbg("clear_stats(b) size=%d\n", g_hash_table_size(st->valid));
+    //g_hash_table_foreach(st->valid, enum_valid, "UPDATEB");
 	g_hash_table_foreach_remove(st->valid, zg_hash_free_key, NULL);
+    //dbg("clear_stats(e) size=%d\n", g_hash_table_size(st->valid));
+    //g_hash_table_foreach(st->valid, enum_valid, "UPDATEE");
     g_hash_table_foreach_remove(st->hours, free_gpointer_item, NULL);
     
     if (st->odxcall)     {g_free(st->odxcall);     st->odxcall      = NULL;}
@@ -162,14 +173,14 @@ void clear_stats(struct stats *st){
     if (st->odxoperator) {g_free(st->odxoperator); st->odxoperator  = NULL;}
     st->odxqrb_int = 0;
 
-    dxi = get_dw_item_by_call(dw, ctest->pcall);
+	dxi = get_dw_item_by_call(dw, ctest->pcall);
     if (!dxi) zinternal("unknown dxcc of my callsign");
 
     zg_free0(st->mycontinent);
     st->mycontinent = g_strdup(dxi->continent);
     zg_free0(st->mydxcc);
     st->mydxcc = g_strdup(get_dxcc(dw, s, ctest->pcall));
-
+	//dbg("*** clear_stats(%c, %s) st=%p    st->mydxcc = %s\n", st->bandchar, from, st, st->mydxcc);
 	st->oldphase = 1;
 }
 
@@ -214,6 +225,7 @@ int find_preferred_dxc(int type, char *dxc){
 /* set and dirty */
 #define SAD_INT(item,value) {if (q->item!=value) band->dirty_save=1;q->item=value;}
 
+
 void update_stats(struct band *band, struct stats *st, struct qso *q){
     gint date, qsop, anew;
     gchar s[120], *c;
@@ -236,7 +248,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
     }
     if (band->stats==st) MUTEX_LOCK(band->stats);
     
-/*    dbg("update_stats(%p, %c, %p)\n", st, b->bandchar, q);*/
+    //dbg("update_stats(%p, %c, %p)\n", st, band->bandchar, q);
     if (!q->error){
         date = atoi(q->date_str);
         if (date < st->first_date) st->first_date = date;
@@ -258,15 +270,22 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
     
 	char *valid_key = stats_valid_key(s, q->callsign, q->locator, q->phase, q->mode);
     
-    dbg("update_stats(%s)\n", valid_key);    
+    //dbg("update_stats(%s) size=%d\n", valid_key, g_hash_table_size(st->valid));
+    //g_hash_table_foreach(st->valid, enum_valid, "UPDATE");
 
-	if (g_hash_table_lookup_extended(st->valid, valid_key, &orig_key, (gpointer*)&orig_value) && ctest->expmode == 0){
-        SAD_INT(dupe,1);
+    if (ctest->expmode == 0){
+        if (g_hash_table_lookup_extended(st->valid, valid_key, &orig_key, (gpointer*)&orig_value)){
+            SAD_INT(dupe,1);
+        }else{
+            char *v = g_strdup(valid_key);
+            //dbg("v=%p %s\n", v, v);
+            g_hash_table_insert(st->valid, v, q);
+            SAD_INT(dupe,0);
+        }
     }else{
-		g_hash_table_insert(st->valid, g_strdup(valid_key), q);
         SAD_INT(dupe,0);
     }
-    
+
     switch (ctest->qsop_method){
         case 1: /* rounded distance */
 			qsop = iaru_round(q->qrb) * ctest->qsomult * band->qsomultb;
@@ -305,7 +324,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
             break;
         case 13:
 //            dbg("call=%s \n", q->callsign);
-            dwi = get_dw_item_by_call(dw, q->callsign);
+			dwi = get_dw_item_by_call(dw, q->callsign);
             if (!dwi){
 //                dbg("Don't know dxcc of %s\n", ctest->pcall);
                 qsop=0;              
@@ -329,6 +348,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
             break;
         case 14:
 //            dbg("call=%s \n", q->callsign);
+			//dbg("st=%p   %c%s  call=%s mydxcc=%s\n", st, st->bandchar, q->qsonrs, q->callsign, st->mydxcc);
             dwi = get_dw_item_by_call(dw, q->callsign);
             if (!dwi){
 //                dbg("Don't know dxcc of %s\n", ctest->pcall);
@@ -339,8 +359,9 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
             if (band->qrg_min<10000) qsop = 2; // bellow 10 MHz
             if (strcasecmp(dwi->continent, st->mycontinent)==0){ // same continent
                 c = get_dxcc(dw, s, q->callsign);
-//                dbg("  dxcc=%s  mydxcc=%s\n", c, st->mydxcc);
-                if (c && strcasecmp(c, st->mydxcc)==0){  // same DXCC
+                //dbg("st=%p   %c%s  call=%s dxcc=%s  mydxcc=%s\n", st, st->bandchar, q->qsonrs, q->callsign, c, st->mydxcc);
+				if (c && strcasecmp(c, st->mydxcc) == 0){  // same DXCC
+				//if (c && strcmp(c, st->mydxcc) == 0){  // same DXCC
                     qsop = 1;               // same dxcc 0
                 }else{
 //                    dbg("  cont=%s\n", dwi->continent);
@@ -394,7 +415,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 		case 20: /* WWL ring Contest Lombardia */
 			qsop = (qsopwr(ctest->pwwlo, q->locator) - 1) * ctest->qsomult * band->qsomultb;
 			break;
-        case 21: // R-Contest
+        case 21: // R-Contest, Ruhrgebiets Contest
             qsop = 1;
 			break;
 		case 22: /* Moon contest */
@@ -424,7 +445,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 
     q->unkcall=!find_wwl_by_call(cw, q->callsign);
     q->susploc = get_susp_loc(cw, dw, q->callsign, q->locator) & 0xff;
-    
+
     anew=q->new_;
 
     if (ctest->wwlused){    
@@ -461,7 +482,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 		}
     }
     
- 
+
     get_dxcc(dw, s, q->callsign);
     if (g_hash_table_lookup_extended(st->dxcs, s, &orig_key, (gpointer*) &orig_value)){
         orig_value->count++; 
@@ -490,7 +511,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
         }
     }
         
-        
+
 
     if (ctest->excused > 0 && ctest->excname){
         safe_strncpy0(s, q->exc, 20);
@@ -540,7 +561,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
             orig_value->qso = q;
         }
     }
-    
+
     wwls  = g_hash_table_size(st->wwls);
     dxcs  = g_hash_table_size(st->dxcs);
     prefs = g_hash_table_size(st->prefs);
@@ -593,12 +614,13 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 		q->new_ &= ~NEW_PHASE;
 	}
 
-    
+
 x:;    
-    if (band->stats==st) MUTEX_UNLOCK(band->stats);
+	if (band->stats == st) MUTEX_UNLOCK(band->stats);
 }
 
 void recalc_stats(struct band *band){
+	//dbg("*** recalc_stats(%c) dirty=%d\n", band->bandchar, band->dirty_stats);
     
     if (!band->dirty_stats) return;
     stats_thread_kill(band);
@@ -1495,7 +1517,7 @@ void stats_merge_prefs_fn(gpointer key, gpointer value, gpointer data){
     gpointer orig_key;
     struct cntpts *orig_value;
 
-    dbg("stats_merge_prefs_fn('%s') cp=%d %d\n", key, cp->count, cp->points); 
+    //dbg("stats_merge_prefs_fn('%s') cp=%d %d\n", key, cp->count, cp->points); 
     if (!g_hash_table_lookup_extended(ctest->allb_prefs, pref, &orig_key, (gpointer) &orig_value)){
         orig_value = g_new0(struct cntpts, 1);
         g_hash_table_insert(ctest->allb_prefs, g_strdup(pref), orig_value);
@@ -1601,6 +1623,7 @@ char *stats_valid_key(char *buf, const char *call, const char *wwl, const int ph
 			break;
 		}
 	}
+    //printf("stats_valid_key='%s'\n", buf);
 	return buf;
 }
 
@@ -1634,12 +1657,13 @@ char *stats_valid_key_display(char *buf, const char *call, const char *wwl, cons
 				break;
 		}
 	}
+    //printf("stats_valid_key_display='%s'\n", buf);
 	return buf;
 }
 
 static void dump(gpointer key, gpointer value, gpointer userdata)
 {
-	g_string_append_printf((GString *)userdata, "%s ", key);
+	g_string_append_printf((GString *)userdata, "%s ", (char*)key);
 }
 
 struct qso *stats_get_dupe(struct stats *st, const char *call, const char *wwl, const int phase, const enum modes mode){
