@@ -491,7 +491,9 @@ void free_subwin(struct subwin *sw){
     if (sw->il) g_free(sw->il);
 	if (sw->adns) zasyncdns_free(sw->adns);
 	if (sw->kstusers) g_hash_table_remove_all(sw->kstusers);
+	g_free(sw->acunder);
 	g_free(sw->callunder);
+	g_free(sw->refunder);
 	zg_ptr_array_free_all(sw->callsunder);
 	g_free(sw->pattern);
 	g_free(sw->player_filename);
@@ -517,6 +519,20 @@ struct subwin *sw_raise_or_new(enum sw_type type){
 	return swnew;
 }
 
+// in-place replace &amp; -> &
+static void htmlchars(char *str){
+	char *s = str;
+	while (1){
+		char *c = strchr(s, '&');
+		if (c == NULL) return;
+
+		if (strncasecmp(c + 1, "amp;", 4) == 0){
+			memmove(c + 1, c + 5, strlen(c) - 4);
+		}
+		s++;
+	}
+}
+
 int sw_add_line(struct subwin *sw, gchar *line, int eol){
     int len, a, ret;
     gchar *c, *last, *str, fill[10];
@@ -541,6 +557,9 @@ int sw_add_line(struct subwin *sw, gchar *line, int eol){
         g_free(last);
     }
 
+	if (sw->type == SWT_KST){
+		htmlchars(str);
+	}
     
 	if (eol){
 		ret = dxc_read_spot(str);
@@ -2022,7 +2041,7 @@ void sw_toggle_highlight(struct subwin *sw, gchar *s){
     /* DX de PA4PS:    144145.0  K6MYC        Tnx eme new# best-20           0841Z */
     c1=NULL;
     c2=NULL;
-    ret=regmatch(c, "^DX DE [0-9A-Z]+:..............([0-9A-Z\\/]+)", &c1, &c2, NULL);
+    ret=regmatch(c, "^DX [Dd][Ee] [0-9A-Z/]+:..............([0-9A-Z\\/]+)", &c1, &c2, NULL);
     if (ret==0){
         safe_strncpy0(call, c2, 19);
         //text = "@DXC";
