@@ -97,7 +97,6 @@ struct subwin {
     int mx, my, ch_mouse_value;
 	int showlines, showwwls;
 #endif    
-    double myw, myh;
     GPtrArray *chbands; // of struct chband
 #ifdef HAVE_SNDFILE
     int pl_rate;
@@ -118,6 +117,9 @@ struct subwin {
 	int chat; // unix chat, not KST chat
 	int first_shus;
 	char *callunder;
+	char *refunder;
+	char *acunder;
+	int uhel;
 	GPtrArray* callsunder;
 	int callunderx, callundery;
 	double qrg; // kHz
