@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2020  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2021  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@
 #include "tsdl.h"
 #include "zstring.h"
 
+
+
 #define NOTHING 1
 static int rigerr = NOTHING;
 
@@ -93,7 +95,7 @@ void init_trigs(void){
         verbose |= crig->rig_verbose;
     }
     if (verbose)
-        rig_set_debug(RIG_DEBUG_VERBOSE);
+		rig_set_debug(RIG_DEBUG_TRACE);
     else
         rig_set_debug(RIG_DEBUG_NONE);
 
@@ -101,7 +103,7 @@ void init_trigs(void){
         riglist = z_ptr_array_new();
 		//if (!rig_load_all_backends_called){
 			rig_load_all_backends();
-		//	rig_load_all_backends_called = 1;
+			//	rig_load_all_backends_called = 1;
 		//}
         rig_list_foreach(trig_save_riglist, riglist);
         z_ptr_array_qsort(riglist, trig_compare);
@@ -501,7 +503,7 @@ gpointer trig_main(gpointer xxx){
         return NULL;
     }
 
-    safe_strncpy(trig->rig->state.rigport.pathname, trig->filename, FILPATHLEN);
+    safe_strncpy(trig->rig->state.rigport.pathname, trig->filename, HAMLIB_FILPATHLEN);
     if (trig->speed > 0) trig->rig->state.rigport.parm.serial.rate = trig->speed;
 	if (trig->handshake_none) trig->rig->state.rigport.parm.serial.handshake = RIG_HANDSHAKE_NONE;
 
