@@ -579,6 +579,7 @@ int sdl_setvideomode(int w, int h, int init_x){
     sdl->yellow2=z_makecol(224,224,0);
     sdl->green=z_makecol(0,255,0);
     sdl->red=z_makecol(255,0,0);
+	sdl->magenta = z_makecol(255, 0, 255);
 
 	sdl->event_thread_break=0;
     //dbg("creating SDL thread\n");
@@ -976,6 +977,7 @@ gpointer sdl_event_thread(gpointer handle){
     oldmtv.tv_sec = 0;
     oldmtv.tv_usec = 0;
 
+    memset(&ev, 0, sizeof(struct event));
     
 	fd = GPOINTER_TO_INT(handle);
     //dbg("SDL thread is alive\n");
@@ -1032,11 +1034,16 @@ gpointer sdl_event_thread(gpointer handle){
 #endif
 
 		
-        
+        char type[10];
+        strcpy(type, "?");
+
         switch(sev.type){
 #ifdef Z_HAVE_SDL2
 			case SDL_TEXTINPUT:
-				//dbg("TEXTINPUT: '%s'\n", sev.text.text);
+                strcpy(type, "TEXTINPUT");
+				SDL_Keymod mod = SDL_GetModState();
+				dbg("TEXTINPUT: '%s'%s\n", sev.text.text, (mod & KMOD_ALT) ? " +ALT" : "");
+				if (mod & KMOD_ALT) break;
 				ev.ev = EV_KBD;
 				if (handle_unicode(&sev, &ev)) goto sendkeydown;
 
@@ -1044,9 +1051,10 @@ gpointer sdl_event_thread(gpointer handle){
 				break;
 #endif
             case SDL_KEYDOWN:
+                strcpy(type, "KEYDOWN");
 #if 1
 				//if (sev.key.keysym.sym != SDLK_LCTRL && sev.key.keysym.sym != SDLK_RCTRL && sev.key.keysym.sym != SDLK_LALT && sev.key.keysym.sym != SDLK_RALT && sev.key.keysym.sym != SDLK_LSHIFT && sev.key.keysym.sym != SDLK_RSHIFT)
-					//dbg("KEYDOWN: scan=%d sym=%d '%c' mod=%d (%s)   GetMods=(%s)\n", sev.key.keysym.scancode, sev.key.keysym.sym, sev.key.keysym.sym, sev.key.keysym.mod, dumpmods(&sev.key.keysym), dumpmods(NULL));
+				dbg("KEYDOWN: scan=%d sym=%d '%c' mod=%d (%s)   GetMods=(%s)\n", sev.key.keysym.scancode, sev.key.keysym.sym, sev.key.keysym.sym, sev.key.keysym.mod, dumpmods(&sev.key.keysym), dumpmods(NULL));
 #endif                
 
 				if (cfg->altsyms){
@@ -1375,6 +1383,7 @@ gpointer sdl_event_thread(gpointer handle){
         continue;
 sendkeydown:        
         //dbg("cfg->altsyms=%d sdl->altsym=%d ev->x=%c\n", cfg->altsyms, sdl->altsym, ev.x);
+        dbg("SEND %s\n", type);
         if (oldaltsym){
             int oldk = ev.x;
             switch (ev.x){
